# -*- coding: utf-8 -*-

"""
Set up the Salt multimaster test suite
"""

# Import Python libs
from __future__ import absolute_import, print_function

import copy
import logging
import os
import shutil
import stat
import sys
import threading
import time

import salt.config
import salt.log.setup as salt_log_setup
import salt.utils.path
import salt.utils.platform
from salt.utils.immutabletypes import freeze
from salt.utils.verify import verify_env

# Import Salt libs
from tests.integration import (
    SocketServerRequestHandler,
    TestDaemon,
    TestDaemonStartFailed,
    ThreadedSocketServer,
    get_unused_localhost_port,
)
from tests.support.parser import PNUM, print_header

# Import salt tests support dirs
from tests.support.paths import (
    ENGINES_DIR,
    FILES,
    INTEGRATION_TEST_DIR,
    LOG_HANDLERS_DIR,
    SCRIPT_DIR,
    TMP,
)

# Import salt tests support libs
from tests.support.processes import SaltMaster, SaltMinion, start_daemon
from tests.support.runtests import RUNTIME_VARS

try:
    from salt.utils.odict import OrderedDict
except ImportError:
    from collections import OrderedDict

log = logging.getLogger(__name__)


SALT_LOG_PORT = get_unused_localhost_port()


class MultimasterTestDaemon(TestDaemon):
    """
    Set up the master and minion daemons, and run related cases
    """

    def __enter__(self):
        """
        Start a master and minion
        """
        # Setup the multiprocessing logging queue listener
        salt_log_setup.setup_multiprocessing_logging_listener(self.mm_master_opts)

        # Set up PATH to mockbin
        self._enter_mockbin()

        self.master_targets = [self.mm_master_opts, self.mm_sub_master_opts]
        self.minion_targets = set(["mm-minion", "mm-sub-minion"])

        if self.parser.options.transport == "zeromq":
            self.start_zeromq_daemons()
        elif self.parser.options.transport == "raet":
            self.start_raet_daemons()
        elif self.parser.options.transport == "tcp":
            self.start_tcp_daemons()

        self.pre_setup_minions()
        self.setup_minions()

        # if getattr(self.parser.options, 'ssh', False):
        # self.prep_ssh()

        self.wait_for_minions(time.time(), self.MINIONS_CONNECT_TIMEOUT)

        if self.parser.options.sysinfo:
            try:
                print_header(
                    "~~~~~~~ Versions Report ",
                    inline=True,
                    width=getattr(self.parser.options, "output_columns", PNUM),
                )
            except TypeError:
                print_header("~~~~~~~ Versions Report ", inline=True)

            print("\n".join(salt.version.versions_report()))

            try:
                print_header(
                    "~~~~~~~ Minion Grains Information ",
                    inline=True,
                    width=getattr(self.parser.options, "output_columns", PNUM),
                )
            except TypeError:
                print_header("~~~~~~~ Minion Grains Information ", inline=True)

            grains = self.client.cmd("minion", "grains.items")

            minion_opts = self.mm_minion_opts.copy()
            minion_opts["color"] = self.parser.options.no_colors is False
            salt.output.display_output(grains, "grains", minion_opts)

        try:
            print_header(
                "=",
                sep="=",
                inline=True,
                width=getattr(self.parser.options, "output_columns", PNUM),
            )
        except TypeError:
            print_header("", sep="=", inline=True)

        try:
            return self
        finally:
            self.post_setup_minions()

    def __exit__(self, type, value, traceback):
        """
        Kill the minion and master processes
        """
        try:
            if hasattr(self.sub_minion_process, "terminate"):
                self.sub_minion_process.terminate()
            else:
                log.error("self.sub_minion_process can't be terminate.")
        except AttributeError:
            pass

        try:
            if hasattr(self.minion_process, "terminate"):
                self.minion_process.terminate()
            else:
                log.error("self.minion_process can't be terminate.")
        except AttributeError:
            pass

        try:
            if hasattr(self.sub_master_process, "terminate"):
                self.sub_master_process.terminate()
            else:
                log.error("self.sub_master_process can't be terminate.")
        except AttributeError:
            pass

        try:
            if hasattr(self.master_process, "terminate"):
                self.master_process.terminate()
            else:
                log.error("self.master_process can't be terminate.")
        except AttributeError:
            pass

        self._exit_mockbin()
        self._exit_ssh()
        # Shutdown the multiprocessing logging queue listener
        salt_log_setup.shutdown_multiprocessing_logging()
        salt_log_setup.shutdown_multiprocessing_logging_listener(daemonizing=True)
        # Shutdown the log server
        self.log_server.shutdown()
        self.log_server.server_close()
        self.log_server_process.join()

    def start_zeromq_daemons(self):
        """
        Fire up the daemons used for zeromq tests
        """
        self.log_server = ThreadedSocketServer(
            ("localhost", SALT_LOG_PORT), SocketServerRequestHandler
        )
        self.log_server_process = threading.Thread(target=self.log_server.serve_forever)
        self.log_server_process.start()
        try:
            sys.stdout.write(
                " * {LIGHT_YELLOW}Starting salt-master ... {ENDC}".format(**self.colors)
            )
            sys.stdout.flush()
            self.master_process = start_daemon(
                daemon_name="salt-master",
                daemon_id=self.mm_master_opts["id"],
                daemon_log_prefix="salt-master/{}".format(self.mm_master_opts["id"]),
                daemon_cli_script_name="master",
                daemon_config=self.mm_master_opts,
                daemon_config_dir=RUNTIME_VARS.TMP_MM_CONF_DIR,
                daemon_class=SaltMaster,
                bin_dir_path=SCRIPT_DIR,
                fail_hard=True,
                start_timeout=120,
            )
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_GREEN}Starting salt-master ... STARTED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
        except (RuntimeWarning, RuntimeError):
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_RED}Starting salt-master ... FAILED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            raise TestDaemonStartFailed()

        # Clone the master key to sub-master's pki dir
        for keyfile in ("master.pem", "master.pub"):
            shutil.copyfile(
                os.path.join(self.mm_master_opts["pki_dir"], keyfile),
                os.path.join(self.mm_sub_master_opts["pki_dir"], keyfile),
            )

        try:
            sys.stdout.write(
                " * {LIGHT_YELLOW}Starting second salt-master ... {ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            self.sub_master_process = start_daemon(
                daemon_name="sub salt-master",
                daemon_id=self.mm_master_opts["id"],
                daemon_log_prefix="sub-salt-master/{}".format(
                    self.mm_sub_master_opts["id"]
                ),
                daemon_cli_script_name="master",
                daemon_config=self.mm_sub_master_opts,
                daemon_config_dir=RUNTIME_VARS.TMP_MM_SUB_CONF_DIR,
                daemon_class=SaltMaster,
                bin_dir_path=SCRIPT_DIR,
                fail_hard=True,
                start_timeout=120,
            )
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_GREEN}Starting second salt-master ... STARTED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
        except (RuntimeWarning, RuntimeError):
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_RED}Starting second salt-master ... FAILED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            raise TestDaemonStartFailed()

        try:
            sys.stdout.write(
                " * {LIGHT_YELLOW}Starting salt-minion ... {ENDC}".format(**self.colors)
            )
            sys.stdout.flush()
            self.minion_process = start_daemon(
                daemon_name="salt-minion",
                daemon_id=self.mm_master_opts["id"],
                daemon_log_prefix="salt-minion/{}".format(self.mm_minion_opts["id"]),
                daemon_cli_script_name="minion",
                daemon_config=self.mm_minion_opts,
                daemon_config_dir=RUNTIME_VARS.TMP_MM_CONF_DIR,
                daemon_class=SaltMinion,
                bin_dir_path=SCRIPT_DIR,
                fail_hard=True,
                start_timeout=120,
            )
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_GREEN}Starting salt-minion ... STARTED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
        except (RuntimeWarning, RuntimeError):
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_RED}Starting salt-minion ... FAILED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            raise TestDaemonStartFailed()

        try:
            sys.stdout.write(
                " * {LIGHT_YELLOW}Starting sub salt-minion ... {ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            self.sub_minion_process = start_daemon(
                daemon_name="sub salt-minion",
                daemon_id=self.mm_master_opts["id"],
                daemon_log_prefix="sub-salt-minion/{}".format(
                    self.mm_sub_minion_opts["id"]
                ),
                daemon_cli_script_name="minion",
                daemon_config=self.mm_sub_minion_opts,
                daemon_config_dir=RUNTIME_VARS.TMP_MM_SUB_CONF_DIR,
                daemon_class=SaltMinion,
                bin_dir_path=SCRIPT_DIR,
                fail_hard=True,
                start_timeout=120,
            )
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_GREEN}Starting sub salt-minion ... STARTED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
        except (RuntimeWarning, RuntimeError):
            sys.stdout.write(
                "\r{0}\r".format(
                    " " * getattr(self.parser.options, "output_columns", PNUM)
                )
            )
            sys.stdout.write(
                " * {LIGHT_RED}Starting sub salt-minion ... FAILED!\n{ENDC}".format(
                    **self.colors
                )
            )
            sys.stdout.flush()
            raise TestDaemonStartFailed()

    start_tcp_daemons = start_zeromq_daemons

    def wait_for_minions(self, start, timeout, sleep=5):
        """
        Ensure all minions and masters (including sub-masters) are connected.
        """
        success = [False] * len(self.master_targets)
        while True:
            for num, client in enumerate(self.clients):
                if success[num]:
                    continue
                try:
                    ret = self.client.run_job("*", "test.ping")
                except salt.exceptions.SaltClientError:
                    ret = None
                if ret and "minions" not in ret:
                    continue
                if ret and sorted(ret["minions"]) == sorted(self.minion_targets):
                    success[num] = True
                    continue
            if all(success):
                break
            if time.time() - start >= timeout:
                raise RuntimeError("Ping Minions Failed")
            time.sleep(sleep)

    @property
    def clients(self):
        """
        Return a local client which will be used for example to ping and sync
        the test minions.

        This client is defined as a class attribute because its creation needs
        to be deferred to a latter stage. If created it on `__enter__` like it
        previously was, it would not receive the master events.
        """
        if "runtime_clients" not in RUNTIME_VARS.RUNTIME_CONFIGS:
            RUNTIME_VARS.RUNTIME_CONFIGS["runtime_clients"] = OrderedDict()

        runtime_clients = RUNTIME_VARS.RUNTIME_CONFIGS["runtime_clients"]
        for mopts in self.master_targets:
            if mopts["id"] in runtime_clients:
                continue
            runtime_clients[mopts["id"]] = salt.client.get_local_client(mopts=mopts)
        return runtime_clients

    @property
    def client(self):
        return self.clients["mm-master"]

    @classmethod
    def transplant_configs(cls, transport="zeromq"):
        os.makedirs(RUNTIME_VARS.TMP_MM_CONF_DIR)
        os.makedirs(RUNTIME_VARS.TMP_MM_SUB_CONF_DIR)
        print(
            " * Transplanting multimaster configuration files to '{0}'".format(
                RUNTIME_VARS.TMP_CONF_DIR
            )
        )
        tests_known_hosts_file = os.path.join(
            RUNTIME_VARS.TMP_CONF_DIR, "salt_ssh_known_hosts"
        )

        # Primary master in multimaster environment
        master_opts = salt.config._read_conf_file(
            os.path.join(RUNTIME_VARS.CONF_DIR, "master")
        )
        master_opts.update(
            salt.config._read_conf_file(
                os.path.join(RUNTIME_VARS.CONF_DIR, "mm_master")
            )
        )
        master_opts["known_hosts_file"] = tests_known_hosts_file
        master_opts["cachedir"] = "cache"
        master_opts["user"] = RUNTIME_VARS.RUNNING_TESTS_USER
        master_opts["config_dir"] = RUNTIME_VARS.TMP_MM_CONF_DIR
        master_opts["root_dir"] = os.path.join(TMP, "rootdir-multimaster")
        master_opts["pki_dir"] = "pki"
        file_tree = {
            "root_dir": os.path.join(FILES, "pillar", "base", "file_tree"),
            "follow_dir_links": False,
            "keep_newline": True,
        }
        master_opts["ext_pillar"].append({"file_tree": file_tree})

        # Secondary master in multimaster environment
        sub_master_opts = salt.config._read_conf_file(
            os.path.join(RUNTIME_VARS.CONF_DIR, "master")
        )
        sub_master_opts.update(
            salt.config._read_conf_file(
                os.path.join(RUNTIME_VARS.CONF_DIR, "mm_sub_master")
            )
        )
        sub_master_opts["known_hosts_file"] = tests_known_hosts_file
        sub_master_opts["cachedir"] = "cache"
        sub_master_opts["user"] = RUNTIME_VARS.RUNNING_TESTS_USER
        sub_master_opts["config_dir"] = RUNTIME_VARS.TMP_MM_SUB_CONF_DIR
        sub_master_opts["root_dir"] = os.path.join(TMP, "rootdir-sub-multimaster")
        sub_master_opts["pki_dir"] = "pki"
        sub_master_opts["ext_pillar"].append({"file_tree": copy.deepcopy(file_tree)})

        # Under windows we can't seem to properly create a virtualenv off of another
        # virtualenv, we can on linux but we will still point to the virtualenv binary
        # outside the virtualenv running the test suite, if that's the case.
        try:
            real_prefix = sys.real_prefix
            # The above attribute exists, this is a virtualenv
            if salt.utils.platform.is_windows():
                virtualenv_binary = os.path.join(
                    real_prefix, "Scripts", "virtualenv.exe"
                )
            else:
                # We need to remove the virtualenv from PATH or we'll get the virtualenv binary
                # from within the virtualenv, we don't want that
                path = os.environ.get("PATH")
                if path is not None:
                    path_items = path.split(os.pathsep)
                    for item in path_items[:]:
                        if item.startswith(sys.base_prefix):
                            path_items.remove(item)
                    os.environ["PATH"] = os.pathsep.join(path_items)
                virtualenv_binary = salt.utils.path.which("virtualenv")
                if path is not None:
                    # Restore previous environ PATH
                    os.environ["PATH"] = path
                if not virtualenv_binary.startswith(real_prefix):
                    virtualenv_binary = None
            if virtualenv_binary and not os.path.exists(virtualenv_binary):
                # It doesn't exist?!
                virtualenv_binary = None
        except AttributeError:
            # We're not running inside a virtualenv
            virtualenv_binary = None

        # This minion connects to both masters
        minion_opts = salt.config._read_conf_file(
            os.path.join(RUNTIME_VARS.CONF_DIR, "minion")
        )
        minion_opts.update(
            salt.config._read_conf_file(
                os.path.join(RUNTIME_VARS.CONF_DIR, "mm_minion")
            )
        )
        minion_opts["cachedir"] = "cache"
        minion_opts["user"] = RUNTIME_VARS.RUNNING_TESTS_USER
        minion_opts["config_dir"] = RUNTIME_VARS.TMP_MM_CONF_DIR
        minion_opts["root_dir"] = os.path.join(TMP, "rootdir-multimaster")
        minion_opts["pki_dir"] = "pki"
        minion_opts["hosts.file"] = os.path.join(TMP, "rootdir", "hosts")
        minion_opts["aliases.file"] = os.path.join(TMP, "rootdir", "aliases")
        if virtualenv_binary:
            minion_opts["venv_bin"] = virtualenv_binary

        # This sub_minion also connects to both masters
        sub_minion_opts = salt.config._read_conf_file(
            os.path.join(RUNTIME_VARS.CONF_DIR, "sub_minion")
        )
        sub_minion_opts.update(
            salt.config._read_conf_file(
                os.path.join(RUNTIME_VARS.CONF_DIR, "mm_sub_minion")
            )
        )
        sub_minion_opts["cachedir"] = "cache"
        sub_minion_opts["user"] = RUNTIME_VARS.RUNNING_TESTS_USER
        sub_minion_opts["config_dir"] = RUNTIME_VARS.TMP_MM_SUB_CONF_DIR
        sub_minion_opts["root_dir"] = os.path.join(TMP, "rootdir-sub-multimaster")
        sub_minion_opts["pki_dir"] = "pki"
        sub_minion_opts["hosts.file"] = os.path.join(TMP, "rootdir", "hosts")
        sub_minion_opts["aliases.file"] = os.path.join(TMP, "rootdir", "aliases")
        if virtualenv_binary:
            sub_minion_opts["venv_bin"] = virtualenv_binary

        if transport == "raet":
            master_opts["transport"] = "raet"
            master_opts["raet_port"] = 64506
            sub_master_opts["transport"] = "raet"
            sub_master_opts["raet_port"] = 64556
            minion_opts["transport"] = "raet"
            minion_opts["raet_port"] = 64510
            sub_minion_opts["transport"] = "raet"
            sub_minion_opts["raet_port"] = 64520
            # syndic_master_opts['transport'] = 'raet'

        if transport == "tcp":
            master_opts["transport"] = "tcp"
            sub_master_opts["transport"] = "tcp"
            minion_opts["transport"] = "tcp"
            sub_minion_opts["transport"] = "tcp"

        # Set up config options that require internal data
        master_opts["pillar_roots"] = sub_master_opts["pillar_roots"] = {
            "base": [
                RUNTIME_VARS.TMP_PILLAR_TREE,
                os.path.join(FILES, "pillar", "base"),
            ]
        }
        minion_opts["pillar_roots"] = {
            "base": [
                RUNTIME_VARS.TMP_PILLAR_TREE,
                os.path.join(FILES, "pillar", "base"),
            ]
        }
        master_opts["file_roots"] = sub_master_opts["file_roots"] = {
            "base": [
                os.path.join(FILES, "file", "base"),
                # Let's support runtime created files that can be used like:
                #   salt://my-temp-file.txt
                RUNTIME_VARS.TMP_STATE_TREE,
            ],
            # Alternate root to test __env__ choices
            "prod": [
                os.path.join(FILES, "file", "prod"),
                RUNTIME_VARS.TMP_PRODENV_STATE_TREE,
            ],
        }
        minion_opts["file_roots"] = {
            "base": [
                os.path.join(FILES, "file", "base"),
                # Let's support runtime created files that can be used like:
                #   salt://my-temp-file.txt
                RUNTIME_VARS.TMP_STATE_TREE,
            ],
            # Alternate root to test __env__ choices
            "prod": [
                os.path.join(FILES, "file", "prod"),
                RUNTIME_VARS.TMP_PRODENV_STATE_TREE,
            ],
        }
        master_opts.setdefault("reactor", []).append(
            {"salt/minion/*/start": [os.path.join(FILES, "reactor-sync-minion.sls")]}
        )
        for opts_dict in (master_opts, sub_master_opts):
            if "ext_pillar" not in opts_dict:
                opts_dict["ext_pillar"] = []
            if salt.utils.platform.is_windows():
                opts_dict["ext_pillar"].append(
                    {"cmd_yaml": "type {0}".format(os.path.join(FILES, "ext.yaml"))}
                )
            else:
                opts_dict["ext_pillar"].append(
                    {"cmd_yaml": "cat {0}".format(os.path.join(FILES, "ext.yaml"))}
                )

        # all read, only owner write
        autosign_file_permissions = (
            stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH | stat.S_IWUSR
        )
        for opts_dict in (master_opts, sub_master_opts):
            # We need to copy the extension modules into the new master root_dir or
            # it will be prefixed by it
            new_extension_modules_path = os.path.join(
                opts_dict["root_dir"], "extension_modules"
            )
            if not os.path.exists(new_extension_modules_path):
                shutil.copytree(
                    os.path.join(INTEGRATION_TEST_DIR, "files", "extension_modules"),
                    new_extension_modules_path,
                )
            opts_dict["extension_modules"] = os.path.join(
                opts_dict["root_dir"], "extension_modules"
            )

            # Copy the autosign_file to the new  master root_dir
            new_autosign_file_path = os.path.join(
                opts_dict["root_dir"], "autosign_file"
            )
            shutil.copyfile(
                os.path.join(INTEGRATION_TEST_DIR, "files", "autosign_file"),
                new_autosign_file_path,
            )
            os.chmod(new_autosign_file_path, autosign_file_permissions)

        # Point the config values to the correct temporary paths
        for name in ("hosts", "aliases"):
            optname = "{0}.file".format(name)
            optname_path = os.path.join(TMP, name)
            master_opts[optname] = optname_path
            sub_master_opts[optname] = optname_path
            minion_opts[optname] = optname_path
            sub_minion_opts[optname] = optname_path

        master_opts["runtests_conn_check_port"] = get_unused_localhost_port()
        sub_master_opts["runtests_conn_check_port"] = get_unused_localhost_port()
        minion_opts["runtests_conn_check_port"] = get_unused_localhost_port()
        sub_minion_opts["runtests_conn_check_port"] = get_unused_localhost_port()

        for conf in (master_opts, sub_master_opts, minion_opts, sub_minion_opts):
            if "engines" not in conf:
                conf["engines"] = []
            conf["engines"].append({"salt_runtests": {}})

            if "engines_dirs" not in conf:
                conf["engines_dirs"] = []

            conf["engines_dirs"].insert(0, ENGINES_DIR)

            if "log_handlers_dirs" not in conf:
                conf["log_handlers_dirs"] = []
            conf["log_handlers_dirs"].insert(0, LOG_HANDLERS_DIR)
            conf["runtests_log_port"] = SALT_LOG_PORT
            conf["runtests_log_level"] = (
                os.environ.get("TESTS_MIN_LOG_LEVEL_NAME") or "debug"
            )

        # ----- Transcribe Configuration ---------------------------------------------------------------------------->
        computed_config = copy.deepcopy(master_opts)
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_MM_CONF_DIR, "master"), "w"
        ) as wfh:
            salt.utils.yaml.safe_dump(
                copy.deepcopy(master_opts), wfh, default_flow_style=False
            )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_MM_SUB_CONF_DIR, "master"), "w"
        ) as wfh:
            salt.utils.yaml.safe_dump(
                copy.deepcopy(sub_master_opts), wfh, default_flow_style=False
            )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_MM_CONF_DIR, "minion"), "w"
        ) as wfh:
            salt.utils.yaml.safe_dump(
                copy.deepcopy(minion_opts), wfh, default_flow_style=False
            )
        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_MM_SUB_CONF_DIR, "minion"), "w"
        ) as wfh:
            salt.utils.yaml.safe_dump(
                copy.deepcopy(sub_minion_opts), wfh, default_flow_style=False
            )
        # <---- Transcribe Configuration -----------------------------------------------------------------------------

        # ----- Verify Environment ---------------------------------------------------------------------------------->
        master_opts = salt.config.master_config(
            os.path.join(RUNTIME_VARS.TMP_MM_CONF_DIR, "master")
        )
        sub_master_opts = salt.config.master_config(
            os.path.join(RUNTIME_VARS.TMP_MM_SUB_CONF_DIR, "master")
        )
        minion_opts = salt.config.minion_config(
            os.path.join(RUNTIME_VARS.TMP_MM_CONF_DIR, "minion")
        )
        sub_minion_opts = salt.config.minion_config(
            os.path.join(RUNTIME_VARS.TMP_MM_SUB_CONF_DIR, "minion")
        )

        RUNTIME_VARS.RUNTIME_CONFIGS["mm_master"] = freeze(master_opts)
        RUNTIME_VARS.RUNTIME_CONFIGS["mm_sub_master"] = freeze(sub_master_opts)
        RUNTIME_VARS.RUNTIME_CONFIGS["mm_minion"] = freeze(minion_opts)
        RUNTIME_VARS.RUNTIME_CONFIGS["mm_sub_minion"] = freeze(sub_minion_opts)

        verify_env(
            [
                os.path.join(master_opts["pki_dir"], "minions"),
                os.path.join(master_opts["pki_dir"], "minions_pre"),
                os.path.join(master_opts["pki_dir"], "minions_rejected"),
                os.path.join(master_opts["pki_dir"], "minions_denied"),
                os.path.join(master_opts["cachedir"], "jobs"),
                os.path.join(master_opts["cachedir"], "raet"),
                os.path.join(master_opts["root_dir"], "cache", "tokens"),
                os.path.join(master_opts["pki_dir"], "accepted"),
                os.path.join(master_opts["pki_dir"], "rejected"),
                os.path.join(master_opts["pki_dir"], "pending"),
                os.path.join(master_opts["cachedir"], "raet"),
                os.path.join(sub_master_opts["pki_dir"], "minions"),
                os.path.join(sub_master_opts["pki_dir"], "minions_pre"),
                os.path.join(sub_master_opts["pki_dir"], "minions_rejected"),
                os.path.join(sub_master_opts["pki_dir"], "minions_denied"),
                os.path.join(sub_master_opts["cachedir"], "jobs"),
                os.path.join(sub_master_opts["cachedir"], "raet"),
                os.path.join(sub_master_opts["root_dir"], "cache", "tokens"),
                os.path.join(sub_master_opts["pki_dir"], "accepted"),
                os.path.join(sub_master_opts["pki_dir"], "rejected"),
                os.path.join(sub_master_opts["pki_dir"], "pending"),
                os.path.join(sub_master_opts["cachedir"], "raet"),
                os.path.join(minion_opts["pki_dir"], "accepted"),
                os.path.join(minion_opts["pki_dir"], "rejected"),
                os.path.join(minion_opts["pki_dir"], "pending"),
                os.path.join(minion_opts["cachedir"], "raet"),
                os.path.join(sub_minion_opts["pki_dir"], "accepted"),
                os.path.join(sub_minion_opts["pki_dir"], "rejected"),
                os.path.join(sub_minion_opts["pki_dir"], "pending"),
                os.path.join(sub_minion_opts["cachedir"], "raet"),
                os.path.dirname(master_opts["log_file"]),
                minion_opts["extension_modules"],
                sub_minion_opts["extension_modules"],
                sub_minion_opts["pki_dir"],
                master_opts["sock_dir"],
                sub_master_opts["sock_dir"],
                sub_minion_opts["sock_dir"],
                minion_opts["sock_dir"],
            ],
            RUNTIME_VARS.RUNNING_TESTS_USER,
            root_dir=master_opts["root_dir"],
        )

        cls.mm_master_opts = master_opts
        cls.mm_sub_master_opts = sub_master_opts
        cls.mm_minion_opts = minion_opts
        cls.mm_sub_minion_opts = sub_minion_opts
        # <---- Verify Environment -----------------------------------------------------------------------------------

    @classmethod
    def config_location(cls):
        return (RUNTIME_VARS.TMP_MM_CONF_DIR, RUNTIME_VARS.TMP_MM_SUB_CONF_DIR)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.win_dns_client as win_dns_client

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class WinDnsClientTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the win_dns_client state
    """

    def setup_loader_modules(self):
        return {win_dns_client: {}}

    def test_dns_exists(self):
        """
            Test to configure the DNS server list in the specified interface
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        with patch.dict(win_dns_client.__opts__, {"test": False}):
            ret.update(
                {
                    "changes": {
                        "Servers Added": [],
                        "Servers Removed": [],
                        "Servers Reordered": [],
                    },
                    "comment": "servers entry is not a list !",
                }
            )
            self.assertDictEqual(win_dns_client.dns_exists("salt"), ret)

            mock = MagicMock(return_value=[2, "salt"])
            with patch.dict(
                win_dns_client.__salt__, {"win_dns_client.get_dns_servers": mock}
            ):
                ret.update(
                    {
                        "changes": {},
                        "comment": repr([2, "salt"]) + " are already" " configured",
                        "result": True,
                    }
                )
                self.assertDictEqual(
                    win_dns_client.dns_exists("salt", [2, "salt"]), ret
                )

                mock = MagicMock(side_effect=[False, True, True])
                with patch.dict(
                    win_dns_client.__salt__, {"win_dns_client.add_dns": mock}
                ):
                    ret.update(
                        {
                            "comment": "Failed to add 1 as DNS" " server number 1",
                            "result": False,
                        }
                    )
                    self.assertDictEqual(
                        win_dns_client.dns_exists("salt", [1, "salt"]), ret
                    )

                    mock = MagicMock(return_value=False)
                    with patch.dict(
                        win_dns_client.__salt__, {"win_dns_client.rm_dns": mock}
                    ):
                        ret.update(
                            {
                                "changes": {
                                    "Servers Added": ["a"],
                                    "Servers Removed": [],
                                    "Servers Reordered": [],
                                },
                                "comment": "Failed to remove 2 from DNS" " server list",
                            }
                        )
                        self.assertDictEqual(
                            win_dns_client.dns_exists("salt", ["a"], "a", 1), ret
                        )

                    ret.update(
                        {"comment": "DNS Servers have been updated", "result": True}
                    )
                    self.assertDictEqual(win_dns_client.dns_exists("salt", ["a"]), ret)

    def test_dns_dhcp(self):
        """
            Test to configure the DNS server list from DHCP Server
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(side_effect=["dhcp", "salt", "salt"])
        with patch.dict(
            win_dns_client.__salt__, {"win_dns_client.get_dns_config": mock}
        ):
            ret.update(
                {
                    "comment": "Local Area Connection already configured"
                    " with DNS from DHCP"
                }
            )
            self.assertDictEqual(win_dns_client.dns_dhcp("salt"), ret)

            with patch.dict(win_dns_client.__opts__, {"test": True}):
                ret.update(
                    {
                        "comment": "",
                        "result": None,
                        "changes": {"dns": "configured from DHCP"},
                    }
                )
                self.assertDictEqual(win_dns_client.dns_dhcp("salt"), ret)

            with patch.dict(win_dns_client.__opts__, {"test": False}):
                mock = MagicMock(return_value=True)
                with patch.dict(
                    win_dns_client.__salt__, {"win_dns_client.dns_dhcp": mock}
                ):
                    ret.update({"result": True})
                    self.assertDictEqual(win_dns_client.dns_dhcp("salt"), ret)

    def test_primary_suffix(self):
        """
            Test to configure the global primary DNS suffix of a DHCP client.
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        ret.update({"comment": "'updates' must be a boolean value"})
        self.assertDictEqual(win_dns_client.primary_suffix("salt", updates="a"), ret)

        mock = MagicMock(
            side_effect=[
                {"vdata": "a"},
                {"vdata": False},
                {"vdata": "b"},
                {"vdata": False},
            ]
        )
        with patch.dict(win_dns_client.__utils__, {"reg.read_value": mock}):
            ret.update({"comment": "No changes needed", "result": True})
            self.assertDictEqual(win_dns_client.primary_suffix("salt", "a"), ret)

            mock = MagicMock(return_value=True)
            with patch.dict(win_dns_client.__utils__, {"reg.set_value": mock}):
                ret.update(
                    {
                        "changes": {"new": {"suffix": "a"}, "old": {"suffix": "b"}},
                        "comment": "Updated primary DNS suffix (a)",
                    }
                )
                self.assertDictEqual(win_dns_client.primary_suffix("salt", "a"), ret)



# -*- coding: utf-8 -*-
"""
Test case for the vault execution module
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.modules.vault as vault

# Import Salt libs
from salt.exceptions import CommandExecutionError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.unit import TestCase


class TestVaultModule(LoaderModuleMockMixin, TestCase):
    """
    Test case for the vault execution module
    """

    def setup_loader_modules(self):
        return {
            vault: {
                "__opts__": {
                    "vault": {
                        "url": "http://127.0.0.1",
                        "auth": {"token": "test", "method": "token"},
                    }
                },
                "__grains__": {"id": "test-minion"},
            }
        }

    def test_read_secret_v1(self):
        """
        Test salt.modules.vault.read_secret function
        """
        version = {"v2": False, "data": None, "metadata": None, "type": None}
        mock_version = MagicMock(return_value=version)
        mock_vault = MagicMock()
        mock_vault.return_value.status_code = 200
        mock_vault.return_value.json.return_value = {"data": {"key": "test"}}
        with patch.dict(
            vault.__utils__, {"vault.make_request": mock_vault}
        ), patch.dict(vault.__utils__, {"vault.is_v2": mock_version}):
            vault_return = vault.read_secret("/secret/my/secret")

        self.assertDictEqual(vault_return, {"key": "test"})

    def test_read_secret_v1_key(self):
        """
        Test salt.modules.vault.read_secret function specifying key
        """
        version = {"v2": False, "data": None, "metadata": None, "type": None}
        mock_version = MagicMock(return_value=version)
        mock_vault = MagicMock()
        mock_vault.return_value.status_code = 200
        mock_vault.return_value.json.return_value = {"data": {"key": "somevalue"}}
        with patch.dict(
            vault.__utils__, {"vault.make_request": mock_vault}
        ), patch.dict(vault.__utils__, {"vault.is_v2": mock_version}):
            vault_return = vault.read_secret("/secret/my/secret", "key")

        self.assertEqual(vault_return, "somevalue")

    def test_read_secret_v2(self):
        """
        Test salt.modules.vault.read_secret function for v2 of kv secret backend
        """
        # given path secrets/mysecret generate v2 output
        version = {
            "v2": True,
            "data": "secrets/data/mysecret",
            "metadata": "secrets/metadata/mysecret",
            "type": "kv",
        }
        mock_version = MagicMock(return_value=version)
        mock_vault = MagicMock()
        mock_vault.return_value.status_code = 200
        v2_return = {
            "data": {
                "data": {"akey": "avalue"},
                "metadata": {
                    "created_time": "2018-10-23T20:21:55.042755098Z",
                    "destroyed": False,
                    "version": 13,
                    "deletion_time": "",
                },
            }
        }

        mock_vault.return_value.json.return_value = v2_return
        with patch.dict(
            vault.__utils__, {"vault.make_request": mock_vault}
        ), patch.dict(vault.__utils__, {"vault.is_v2": mock_version}):
            # Validate metadata returned
            vault_return = vault.read_secret("/secret/my/secret", metadata=True)
            self.assertDictContainsSubset({"data": {"akey": "avalue"}}, vault_return)
            # Validate just data returned
            vault_return = vault.read_secret("/secret/my/secret")
            self.assertDictContainsSubset({"akey": "avalue"}, vault_return)

    def test_read_secret_v2_key(self):
        """
        Test salt.modules.vault.read_secret function for v2 of kv secret backend
        with specified key
        """
        # given path secrets/mysecret generate v2 output
        version = {
            "v2": True,
            "data": "secrets/data/mysecret",
            "metadata": "secrets/metadata/mysecret",
            "type": "kv",
        }
        mock_version = MagicMock(return_value=version)
        mock_vault = MagicMock()
        mock_vault.return_value.status_code = 200
        v2_return = {
            "data": {
                "data": {"akey": "avalue"},
                "metadata": {
                    "created_time": "2018-10-23T20:21:55.042755098Z",
                    "destroyed": False,
                    "version": 13,
                    "deletion_time": "",
                },
            }
        }

        mock_vault.return_value.json.return_value = v2_return
        with patch.dict(
            vault.__utils__, {"vault.make_request": mock_vault}
        ), patch.dict(vault.__utils__, {"vault.is_v2": mock_version}):
            vault_return = vault.read_secret("/secret/my/secret", "akey")

        self.assertEqual(vault_return, "avalue")


class VaultDefaultTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for the default argument in the vault module

    NOTE: This test class is crafted such that the vault.make_request call will
    always fail. If you want to add other unit tests, you should put them in a
    separate class.
    """

    def setup_loader_modules(self):
        return {
            vault: {
                "__grains__": {"id": "foo"},
                "__utils__": {
                    "vault.make_request": MagicMock(side_effect=Exception("FAILED")),
                    "vault.is_v2": MagicMock(
                        return_value={
                            "v2": True,
                            "data": "secrets/data/mysecret",
                            "metadata": "secrets/metadata/mysecret",
                            "type": "kv",
                        }
                    ),
                },
            },
        }

    def setUp(self):
        self.path = "foo/bar/"

    def test_read_secret_with_default(self):
        assert vault.read_secret(self.path, default="baz") == "baz"

    def test_read_secret_no_default(self):
        try:
            vault.read_secret(self.path)
        except CommandExecutionError:
            # This is expected
            pass
        else:
            raise Exception("Should have raised a CommandExecutionError")

    def test_list_secrets_with_default(self):
        assert vault.list_secrets(self.path, default=["baz"]) == ["baz"]

    def test_list_secrets_no_default(self):
        try:
            vault.list_secrets(self.path)
        except CommandExecutionError:
            # This is expected
            pass
        else:
            raise Exception("Should have raised a CommandExecutionError")



# -*- coding: utf-8 -*-
"""
unit tests for the alias state
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.alias as alias

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AliasTest(TestCase, LoaderModuleMockMixin):
    """
    Validate the alias state
    """

    def setup_loader_modules(self):
        return {alias: {}}

    def test_present_has_target(self):
        """
        test alias.present has target already
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Alias {0} already present".format(name),
            "changes": {},
            "name": name,
            "result": True,
        }

        has_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {"aliases.has_target": has_target}):
            self.assertEqual(alias.present(name, target), ret)

    def test_present_has_not_target_test(self):
        """
        test alias.present has't got target yet test mode
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Alias {0} -> {1} is set to be added".format(name, target),
            "changes": {},
            "name": name,
            "result": None,
        }

        has_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {"aliases.has_target": has_target}):
            with patch.dict(alias.__opts__, {"test": True}):
                self.assertEqual(alias.present(name, target), ret)

    def test_present_set_target(self):
        """
        test alias.present set target
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Set email alias {0} -> {1}".format(name, target),
            "changes": {"alias": name},
            "name": name,
            "result": True,
        }

        has_target = MagicMock(return_value=False)
        set_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {"aliases.has_target": has_target}):
            with patch.dict(alias.__opts__, {"test": False}):
                with patch.dict(alias.__salt__, {"aliases.set_target": set_target}):
                    self.assertEqual(alias.present(name, target), ret)

    def test_present_set_target_failed(self):
        """
        test alias.present set target failure
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Failed to set alias {0} -> {1}".format(name, target),
            "changes": {},
            "name": name,
            "result": False,
        }

        has_target = MagicMock(return_value=False)
        set_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {"aliases.has_target": has_target}):
            with patch.dict(alias.__opts__, {"test": False}):
                with patch.dict(alias.__salt__, {"aliases.set_target": set_target}):
                    self.assertEqual(alias.present(name, target), ret)

    def test_absent_already_gone(self):
        """
        test alias.absent already gone
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Alias {0} already absent".format(name),
            "changes": {},
            "name": name,
            "result": True,
        }

        get_target = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {"aliases.get_target": get_target}):
            self.assertEqual(alias.absent(name), ret)

    def test_absent_not_gone_test(self):
        """
        test alias.absent already gone test mode
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Alias {0} is set to be removed".format(name),
            "changes": {},
            "name": name,
            "result": None,
        }

        get_target = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {"aliases.get_target": get_target}):
            with patch.dict(alias.__opts__, {"test": True}):
                self.assertEqual(alias.absent(name), ret)

    def test_absent_rm_alias(self):
        """
        test alias.absent remove alias
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Removed alias {0}".format(name),
            "changes": {"alias": name},
            "name": name,
            "result": True,
        }

        get_target = MagicMock(return_value=True)
        rm_alias = MagicMock(return_value=True)
        with patch.dict(alias.__salt__, {"aliases.get_target": get_target}):
            with patch.dict(alias.__opts__, {"test": False}):
                with patch.dict(alias.__salt__, {"aliases.rm_alias": rm_alias}):
                    self.assertEqual(alias.absent(name), ret)

    def test_absent_rm_alias_failed(self):
        """
        test alias.absent remove alias failure
        """
        name = "saltdude"
        target = "dude@saltstack.com"
        ret = {
            "comment": "Failed to remove alias {0}".format(name),
            "changes": {},
            "name": name,
            "result": False,
        }

        get_target = MagicMock(return_value=True)
        rm_alias = MagicMock(return_value=False)
        with patch.dict(alias.__salt__, {"aliases.get_target": get_target}):
            with patch.dict(alias.__opts__, {"test": False}):
                with patch.dict(alias.__salt__, {"aliases.rm_alias": rm_alias}):
                    self.assertEqual(alias.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Unit Tests for functions located in salt.utils.doc.py.
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.utils.doc

# Import Salt Testing libs
from tests.support.unit import TestCase


class DocUtilsTestCase(TestCase):
    """
    Test case for doc util.
    """

    def test_parse_docstring(self):
        test_keystone_str = """Management of Keystone users
                                ============================

                                :depends:   - keystoneclient Python module
                                :configuration: See :py:mod:`salt.modules.keystone` for setup instructions.
"""

        ret = salt.utils.doc.parse_docstring(test_keystone_str)
        expected_dict = {
            "deps": ["keystoneclient"],
            "full": "Management of Keystone users\n                                "
            "============================\n\n                                "
            ":depends:   - keystoneclient Python module\n                                "
            ":configuration: See :py:mod:`salt.modules.keystone` for setup instructions.\n",
        }
        self.assertDictEqual(ret, expected_dict)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
    :copyright: Copyright 2016 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    pytestsalt.salt.log_handlers.pytest_log_handler
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Salt External Logging Handler
"""

# Import python libs
from __future__ import absolute_import

import errno
import logging
import socket
import threading
from multiprocessing import Queue

import salt.log.setup

# Import Salt libs
import salt.utils.msgpack
from salt.ext import six
from salt.utils.platform import is_darwin

log = logging.getLogger(__name__)

__virtualname__ = "runtests_log_handler"


def __virtual__():
    if "runtests_log_port" not in __opts__:
        return False, "'runtests_log_port' not in options"
    if six.PY3:
        return (
            False,
            "runtests external logging handler is temporarily disabled for Python 3 tests",
        )
    return True


def setup_handlers():
    port = __opts__["runtests_log_port"]
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    try:
        sock.connect(("localhost", port))
    except socket.error as exc:
        if exc.errno == errno.ECONNREFUSED:
            log.warning("Failed to connect to log server")
            return
    finally:
        try:
            sock.shutdown(socket.SHUT_RDWR)
        except OSError:
            pass
        sock.close()

    # One million log messages is more than enough to queue.
    # Above that value, if `process_queue` can't process fast enough,
    # start dropping. This will contain a memory leak in case `process_queue`
    # can't process fast enough of in case it can't deliver the log records at all.
    if is_darwin():
        queue_size = 32767
    else:
        queue_size = 10000000
    queue = Queue(queue_size)
    handler = salt.log.setup.QueueHandler(queue)
    level = salt.log.setup.LOG_LEVELS[
        (__opts__.get("runtests_log_level") or "error").lower()
    ]
    handler.setLevel(level)
    process_queue_thread = threading.Thread(target=process_queue, args=(port, queue))
    process_queue_thread.daemon = True
    process_queue_thread.start()
    return handler


def process_queue(port, queue):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    try:
        sock.connect(("localhost", port))
    except socket.error as exc:
        if exc.errno == errno.ECONNREFUSED:
            sock.shutdown(socket.SHUT_RDWR)
            sock.close()
            log.warning("Failed to connect to log server")
            return

    while True:
        try:
            record = queue.get()
            if record is None:
                # A sentinel to stop processing the queue
                break
            # Just log everything, filtering will happen on the main process
            # logging handlers
            sock.sendall(salt.utils.msgpack.dumps(record.__dict__, encoding="utf-8"))
        except (IOError, EOFError, KeyboardInterrupt, SystemExit):
            if hasattr(exc, "errno") and exc.errno != errno.EPIPE:
                log.exception(exc)
            try:
                sock.shutdown(socket.SHUT_RDWR)
                sock.close()
            except (OSError, socket.error):
                pass
            break
        except Exception as exc:  # pylint: disable=broad-except
            log.warning(
                "An exception occurred in the pytest salt logging " "queue thread: %s",
                exc,
                exc_info_on_loglevel=logging.DEBUG,
            )



# -*- coding: utf-8 -*-
"""
Test salt-call --proxyid commands

tests.integration.proxy.test_shell
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
from __future__ import absolute_import, print_function, unicode_literals

import json
import logging
import sys

import salt.ext.six as six
import salt.utils.json as json
from tests.support.case import ShellCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


class ProxyCallerSimpleTestCase(ShellCase):
    """
    Test salt-call --proxyid <proxyid> commands
    """

    RUN_TIMEOUT = 300

    @staticmethod
    def _load_return(ret):
        try:
            return json.loads("\n".join(ret))
        except ValueError:
            log.warning("Failed to JSON decode: '%s'", ret)
            six.reraise(*sys.exc_info())

    @slowTest
    def test_can_it_ping(self):
        """
        Ensure the proxy can ping
        """
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json test.ping",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertEqual(ret["local"], True)

    @slowTest
    def test_list_pkgs(self):
        """
        Package test 1, really just tests that the virtual function capability
        is working OK.
        """
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json pkg.list_pkgs",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertIn("coreutils", ret["local"])
        self.assertIn("apache", ret["local"])
        self.assertIn("redbull", ret["local"])

    @slowTest
    def test_upgrade(self):
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json pkg.upgrade",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertEqual(ret["local"]["coreutils"]["new"], "2.0")
        self.assertEqual(ret["local"]["redbull"]["new"], "1000.99")

    @slowTest
    def test_service_list(self):
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json service.list",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertIn("ntp", ret["local"])

    @slowTest
    def test_service_start(self):
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json service.start samba",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json service.status samba",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertTrue(ret)

    @slowTest
    def test_service_get_all(self):
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json service.get_all",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertIn("samba", ret["local"])

    @slowTest
    def test_grains_items(self):
        ret = self._load_return(
            self.run_call(
                "--proxyid proxytest --out=json grains.items",
                config_dir=RUNTIME_VARS.TMP_PROXY_CONF_DIR,
            )
        )
        self.assertEqual(ret["local"]["kernel"], "proxy")
        self.assertEqual(ret["local"]["kernelrelease"], "proxy")



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_kinesis as boto_kinesis

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoKinesisTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_kinesis
    """

    # 'present' function tests: 1

    maxDiff = None

    def setup_loader_modules(self):
        return {boto_kinesis: {}}

    def test_stream_present(self):
        """
        Test to ensure the kinesis stream exists.
        """
        name = "new_stream"
        retention_hours = 24
        enhanced_monitoring = ["IteratorAgeMilliseconds"]
        different_enhanced_monitoring = ["IncomingBytes"]
        num_shards = 1

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        shards = [
            {
                "ShardId": "shardId-000000000000",
                "HashKeyRange": {"EndingHashKey": "big number", "StartingHashKey": "0"},
                "SequenceNumberRange": {"StartingSequenceNumber": "bigger number"},
            }
        ]
        stream_description = {
            "HasMoreShards": False,
            "RetentionPeriodHours": retention_hours,
            "StreamName": name,
            "Shards": shards,
            "StreamARN": "",
            "EnhancedMonitoring": [{"ShardLevelMetrics": enhanced_monitoring}],
            "StreamStatus": "ACTIVE",
        }

        exists_mock = MagicMock(
            side_effect=[
                {"result": True},
                {"result": False},
                {"result": True},
                {"result": False},
            ]
        )
        get_stream_mock = MagicMock(
            return_value={"result": {"StreamDescription": stream_description}}
        )
        shard_mock = MagicMock(return_value=[0, 0, {"OpenShards": shards}])
        dict_mock = MagicMock(return_value={"result": True})
        mock_bool = MagicMock(return_value=True)
        with patch.dict(
            boto_kinesis.__salt__,
            {
                "boto_kinesis.exists": exists_mock,
                "boto_kinesis.create_stream": dict_mock,
                "boto_kinesis.get_stream_when_active": get_stream_mock,
                "boto_kinesis.get_info_for_reshard": shard_mock,
                "boto_kinesis.num_shards_matches": mock_bool,
            },
        ):
            # already present, no change required
            comt = (
                "Kinesis stream {0} already exists,\n"
                "Kinesis stream {0}: retention hours did not require change, already set at {1},\n"
                "Kinesis stream {0}: enhanced monitoring did not require change, already set at {2},\n"
                "Kinesis stream {0}: did not require resharding, remains at {3} shards".format(
                    name, retention_hours, enhanced_monitoring, num_shards
                )
            )
            ret.update({"comment": comt})
            self.assertDictEqual(
                boto_kinesis.present(
                    name, retention_hours, enhanced_monitoring, num_shards
                ),
                ret,
            )

            with patch.dict(boto_kinesis.__opts__, {"test": True}):
                # not present, test environment (dry run)
                comt = "Kinesis stream {0} would be created".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    boto_kinesis.present(
                        name, retention_hours, enhanced_monitoring, num_shards
                    ),
                    ret,
                )

                # already present, changes required, test environment (dry run)
                comt = (
                    "Kinesis stream {0} already exists,\n"
                    "Kinesis stream {0}: retention hours would be updated to {1},\n"
                    "Kinesis stream {0}: would enable enhanced monitoring for {2},\n"
                    "Kinesis stream {0}: would disable enhanced monitoring for {3},\n"
                    "Kinesis stream {0}: would be resharded from {4} to {5} shards".format(
                        name,
                        retention_hours + 1,
                        different_enhanced_monitoring,
                        enhanced_monitoring,
                        num_shards,
                        num_shards + 1,
                    )
                )
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    boto_kinesis.present(
                        name,
                        retention_hours + 1,
                        different_enhanced_monitoring,
                        num_shards + 1,
                    ),
                    ret,
                )

            # not present, create and configure
            changes = {"new": {"name": name, "num_shards": num_shards}}

            with patch.dict(boto_kinesis.__opts__, {"test": False}):
                comt = (
                    "Kinesis stream {0} successfully created,\n"
                    "Kinesis stream {0}: retention hours did not require change, already set at {1},\n"
                    "Kinesis stream {0}: enhanced monitoring did not require change, already set at {2},\n"
                    "Kinesis stream {0}: did not require resharding, remains at {3} shards".format(
                        name, retention_hours, enhanced_monitoring, num_shards
                    )
                )
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(
                    ret,
                    boto_kinesis.present(
                        name, retention_hours, enhanced_monitoring, num_shards
                    ),
                )

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the Kinesis stream does not exist.
        """
        name = "new_stream"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(
            side_effect=[{"result": False}, {"result": True}, {"result": True}]
        )
        mock_bool = MagicMock(return_value={"result": True})
        with patch.dict(
            boto_kinesis.__salt__,
            {"boto_kinesis.exists": mock, "boto_kinesis.delete_stream": mock_bool},
        ):
            comt = "Kinesis stream {0} does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(boto_kinesis.absent(name), ret)

            with patch.dict(boto_kinesis.__opts__, {"test": True}):
                comt = "Kinesis stream {0} would be deleted".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_kinesis.absent(name), ret)

            changes = {
                "new": "Stream {0} deleted".format(name),
                "old": "Stream {0} exists".format(name),
            }

            with patch.dict(boto_kinesis.__opts__, {"test": False}):
                comt = "Deleted stream {0}".format(name)
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(boto_kinesis.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.config
import salt.states.winrepo as winrepo
import salt.utils.path
from salt.syspaths import BASE_FILE_ROOTS_DIR

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MockRunnerClient(object):
    """
        Mock RunnerClient class
    """

    def __init__(self):
        pass

    class RunnerClient(object):
        """
            Mock RunnerClient class
        """

        def __init__(self, master_config):
            """
                init method
            """

        @staticmethod
        def cmd(arg1, arg2):
            """
                Mock cmd method
            """
            # TODO: Figure out how to have this return an empty dict or a dict
            # TODO: with expected data
            return []


class WinrepoTestCase(TestCase, LoaderModuleMockMixin):
    """
    Validate the winrepo state
    """

    def setup_loader_modules(self):
        patcher = patch("salt.states.winrepo.salt.runner", MockRunnerClient)
        patcher.start()
        self.addCleanup(patcher.stop)
        return {winrepo: {}}

    def test_genrepo(self):
        """
        Test to refresh the winrepo.p file of the repository
        """
        expected = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        mock_config = MagicMock(
            return_value={"winrepo_dir": "salt", "winrepo_cachefile": "abc"}
        )
        mock_stat = MagicMock(return_value=[0, 1, 2, 3, 4, 5, 6, 7, 8])
        mock_empty_list = MagicMock(return_value=[])
        with patch.object(salt.config, "master_config", mock_config), patch.object(
            os, "stat", mock_stat
        ), patch.object(salt.utils.path, "os_walk", mock_empty_list), patch.dict(
            winrepo.__opts__, {"test": True}
        ):
            # With test=True
            expected.update({"comment": "", "result": None})
            self.assertDictEqual(winrepo.genrepo("salt"), expected)

            with patch.dict(winrepo.__opts__, {"test": False}):
                # With test=False
                expected.update({"result": True})
                self.assertDictEqual(winrepo.genrepo("salt"), expected)

                # Now with no changes, existing winrepo.p
                expected.update({"changes": {"winrepo": []}})
                self.assertDictEqual(winrepo.genrepo("salt", True), expected)

    def test_genrepo_no_dir(self):
        """
        Test genrepo when the dir does not exist
        """
        expected = {
            "name": "salt",
            "changes": {},
            "result": False,
            "comment": "{0} is missing".format(
                os.sep.join([BASE_FILE_ROOTS_DIR, "win", "repo"])
            ),
        }
        with patch.dict(winrepo.__opts__, {"test": False}), patch(
            "os.path.exists", MagicMock(return_value=False)
        ):
            ret = winrepo.genrepo("salt")
            self.assertDictEqual(ret, expected)

    def test_genrepo_no_dir_force(self):
        """
        Test genrepo when the dir does not exist and force=True
        """
        expected = {
            "name": "salt",
            "changes": {"winrepo": []},
            "result": True,
            "comment": "",
        }
        with patch.dict(winrepo.__opts__, {"test": False}), patch(
            "os.path.exists", MagicMock(return_value=False)
        ):
            ret = winrepo.genrepo("salt", force=True)
            self.assertDictEqual(ret, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.rdp as rdp

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class RdpTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.rdp
    """

    def setup_loader_modules(self):
        return {rdp: {}}

    # 'enabled' function tests: 1

    def test_enabled(self):
        """
        Test to enable the RDP service and make sure access
        to the RDP port is allowed in the firewall configuration.
        """
        name = "my_service"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock_t = MagicMock(side_effect=[False, False, True])
        mock_f = MagicMock(return_value=False)
        with patch.dict(rdp.__salt__, {"rdp.status": mock_t, "rdp.enable": mock_f}):
            with patch.dict(rdp.__opts__, {"test": True}):
                comt = "RDP will be enabled"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(rdp.enabled(name), ret)

            with patch.dict(rdp.__opts__, {"test": False}):
                ret.update(
                    {
                        "comment": "",
                        "result": False,
                        "changes": {"RDP was enabled": True},
                    }
                )
                self.assertDictEqual(rdp.enabled(name), ret)

                comt = "RDP is enabled"
                ret.update({"comment": comt, "result": True, "changes": {}})
                self.assertDictEqual(rdp.enabled(name), ret)

    # 'disabled' function tests: 1

    def test_disabled(self):
        """
        Test to disable the RDP service.
        """
        name = "my_service"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock = MagicMock(side_effect=[True, True, False])
        mock_t = MagicMock(return_value=True)
        with patch.dict(rdp.__salt__, {"rdp.status": mock, "rdp.disable": mock_t}):
            with patch.dict(rdp.__opts__, {"test": True}):
                comt = "RDP will be disabled"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(rdp.disabled(name), ret)

            with patch.dict(rdp.__opts__, {"test": False}):
                ret.update(
                    {
                        "comment": "",
                        "result": True,
                        "changes": {"RDP was disabled": True},
                    }
                )
                self.assertDictEqual(rdp.disabled(name), ret)

                comt = "RDP is disabled"
                ret.update({"comment": comt, "result": True, "changes": {}})
                self.assertDictEqual(rdp.disabled(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Carlos D. Álvaro <github@cdalvaro.io>`

    tests.unit.returners.test_slack_webhook_return
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Unit tests for the Slack Webhook Returner.
"""

# Import Python libs
from __future__ import absolute_import

# Import Salt libs
import salt.returners.slack_webhook_return as slack_webhook

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class SlackWebhookReturnerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test slack_webhook returner
    """

    _WEBHOOK = "T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX"
    _AUTHOR_ICON = "https://platform.slack-edge.com/img/default_application_icon.png"
    _SHOW_TASKS = True
    _MINION_NAME = "MacPro"

    _RET = {
        "fun_args": ["config.vim"],
        "jid": "20181227105933129338",
        "return": {
            "file_|-vim files present_|-/Users/cdalvaro/_|-recurse": {
                "comment": "The directory /Users/cdalvaro/ is in the correct state",
                "changes": {},
                "name": "/Users/cdalvaro/",
                "start_time": "10:59:52.252830",
                "result": True,
                "duration": 373.25,
                "__run_num__": 3,
                "__sls__": "config.vim",
                "__id__": "vim files present",
            },
            "pkg_|-vim present_|-vim_|-installed": {
                "comment": "All specified packages are already installed",
                "name": "vim",
                "start_time": "10:59:36.830591",
                "result": True,
                "duration": 1280.127,
                "__run_num__": 0,
                "__sls__": "config.vim",
                "changes": {},
                "__id__": "vim present",
            },
            "git_|-salt vim plugin updated_|-https://github.com/saltstack/salt-vim.git_|-latest": {
                "comment": "https://github.com/saltstack/salt-vim.git cloned to /Users/cdalvaro/.vim/pack/git-plugins/start/salt",
                "name": "https://github.com/saltstack/salt-vim.git",
                "start_time": "11:00:01.892757",
                "result": True,
                "duration": 11243.445,
                "__run_num__": 6,
                "__sls__": "config.vim",
                "changes": {
                    "new": "https://github.com/saltstack/salt-vim.git => /Users/cdalvaro/.vim/pack/git-plugins/start/salt",
                    "revision": {
                        "new": "6ca9e3500cc39dd417b411435d58a1b720b331cc",
                        "old": None,
                    },
                },
                "__id__": "salt vim plugin updated",
            },
            "pkg_|-macvim present_|-caskroom/cask/macvim_|-installed": {
                "comment": "The following packages failed to install/update: caskroom/cask/macvim",
                "name": "caskroom/cask/macvim",
                "start_time": "10:59:38.111119",
                "result": False,
                "duration": 14135.45,
                "__run_num__": 1,
                "__sls__": "config.vim",
                "changes": {},
                "__id__": "macvim present",
            },
        },
        "retcode": 2,
        "success": True,
        "fun": "state.apply",
        "id": _MINION_NAME,
        "out": "highstate",
    }

    _EXPECTED_PAYLOAD = {
        u"attachments": [
            {
                u"title": u"Success: False",
                u"color": u"#272727",
                u"text": u"Function: state.apply\nFunction Args: ['config.vim']\nJID: 20181227105933129338\nTotal: 4\nDuration: 27.03 secs",
                u"author_link": u"{}".format(_MINION_NAME),
                u"author_name": u"{}".format(_MINION_NAME),
                u"fallback": u"{} | Failed".format(_MINION_NAME),
                u"author_icon": _AUTHOR_ICON,
            },
            {u"color": u"good", u"title": u"Unchanged: 2"},
            {
                u"color": u"warning",
                u"fields": [
                    {
                        u"short": False,
                        u"value": u"config.vim.sls | salt vim plugin updated",
                    }
                ],
                u"title": u"Changed: 1",
            },
            {
                u"color": u"danger",
                u"fields": [
                    {u"short": False, u"value": u"config.vim.sls | macvim present"}
                ],
                u"title": u"Failed: 1",
            },
        ]
    }

    def setup_loader_modules(self):
        return {
            slack_webhook: {
                "__opts__": {
                    "slack_webhook.webhook": self._WEBHOOK,
                    "slack_webhook.author_icon": self._AUTHOR_ICON,
                    "slack_webhook.success_title": "{id} | Succeeded",
                    "slack_webhook.failure_title": "{id} | Failed",
                    "slack_webhook.show_tasks": self._SHOW_TASKS,
                }
            }
        }

    def test_no_webhook(self):
        """
        Test returner stops if no webhook is defined
        """
        with patch.dict(slack_webhook.__opts__, {"slack_webhook.webhook": ""}):
            self.assertEqual(slack_webhook.returner(self._RET), None)

    def test_returner(self):
        """
        Test to see if the Slack Webhook returner sends a message
        """
        query_ret = {"body": "ok", "status": 200}
        with patch("salt.utils.http.query", return_value=query_ret):
            self.assertTrue(slack_webhook.returner(self._RET))

    def test_generate_payload_for_state_apply(self):
        """
        Test _generate_payload private method
        """
        test_title = "{} | Failed".format(self._MINION_NAME)
        test_report = slack_webhook._generate_report(self._RET, self._SHOW_TASKS)

        custom_grains = slack_webhook.__grains__
        custom_grains["id"] = self._MINION_NAME
        custom_grains["localhost"] = self._MINION_NAME

        with patch.dict(slack_webhook.__grains__, custom_grains):
            test_payload = slack_webhook._generate_payload(
                self._AUTHOR_ICON, test_title, test_report
            )

        self.assertDictEqual(test_payload, self._EXPECTED_PAYLOAD)

    def test_generate_payload_for_test_ping(self):
        """
        Test _generate_payload private method
        """

        test_ping_ret = {
            "jid": "20200124204109195206",
            "return": True,
            "retcode": 0,
            "id": self._MINION_NAME,
            "fun": "test.ping",
            "fun_args": [],
            "success": True,
        }
        expected_payload = {
            "attachments": [
                {
                    "fallback": "{} | Succeeded".format(self._MINION_NAME),
                    "color": "#272727",
                    "author_name": self._MINION_NAME,
                    "author_link": self._MINION_NAME,
                    "author_icon": self._AUTHOR_ICON,
                    "title": "Success: True",
                    "text": "Function: test.ping\nJID: 20200124204109195206\n",
                },
                {"color": "good", "title": "Return: True"},
            ]
        }

        test_title = "{} | Succeeded".format(self._MINION_NAME)
        test_report = slack_webhook._generate_report(test_ping_ret, self._SHOW_TASKS)

        custom_grains = slack_webhook.__grains__
        custom_grains["id"] = self._MINION_NAME
        custom_grains["localhost"] = self._MINION_NAME

        with patch.dict(slack_webhook.__grains__, custom_grains):
            test_payload = slack_webhook._generate_payload(
                self._AUTHOR_ICON, test_title, test_report
            )

        self.assertDictEqual(test_payload, expected_payload)



# -*- coding: utf-8 -*-
"""
Management of Open vSwitch bridges.
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals


def __virtual__():
    """
    Only make these states available if Open vSwitch module is available.
    """
    if "openvswitch.bridge_create" in __salt__:
        return True
    return (False, "openvswitch module could not be loaded")


def present(name):
    """
    Ensures that the named bridge exists, eventually creates it.

    Args:
        name: The name of the bridge.

    """
    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    # Comment and change messages
    comment_bridge_created = "Bridge {0} created.".format(name)
    comment_bridge_notcreated = "Unable to create bridge: {0}.".format(name)
    comment_bridge_exists = "Bridge {0} already exists.".format(name)
    changes_bridge_created = {
        name: {
            "old": "Bridge {0} does not exist.".format(name),
            "new": "Bridge {0} created".format(name),
        }
    }

    bridge_exists = __salt__["openvswitch.bridge_exists"](name)

    # Dry run, test=true mode
    if __opts__["test"]:
        if bridge_exists:
            ret["result"] = True
            ret["comment"] = comment_bridge_exists
        else:
            ret["result"] = None
            ret["comment"] = comment_bridge_created

        return ret

    if bridge_exists:
        ret["result"] = True
        ret["comment"] = comment_bridge_exists
    else:
        bridge_create = __salt__["openvswitch.bridge_create"](name)
        if bridge_create:
            ret["result"] = True
            ret["comment"] = comment_bridge_created
            ret["changes"] = changes_bridge_created
        else:
            ret["result"] = False
            ret["comment"] = comment_bridge_notcreated

    return ret


def absent(name):
    """
    Ensures that the named bridge does not exist, eventually deletes it.

    Args:
        name: The name of the bridge.

    """

    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    # Comment and change messages
    comment_bridge_deleted = "Bridge {0} deleted.".format(name)
    comment_bridge_notdeleted = "Unable to delete bridge: {0}.".format(name)
    comment_bridge_notexists = "Bridge {0} does not exist.".format(name)
    changes_bridge_deleted = {
        name: {
            "old": "Bridge {0} exists.".format(name),
            "new": "Bridge {0} deleted.".format(name),
        }
    }

    bridge_exists = __salt__["openvswitch.bridge_exists"](name)

    # Dry run, test=true mode
    if __opts__["test"]:
        if not bridge_exists:
            ret["result"] = True
            ret["comment"] = comment_bridge_notexists
        else:
            ret["result"] = None
            ret["comment"] = comment_bridge_deleted

        return ret

    if not bridge_exists:
        ret["result"] = True
        ret["comment"] = comment_bridge_notexists
    else:
        bridge_delete = __salt__["openvswitch.bridge_delete"](name)
        if bridge_delete:
            ret["result"] = True
            ret["comment"] = comment_bridge_deleted
            ret["changes"] = changes_bridge_deleted
        else:
            ret["result"] = False
            ret["comment"] = comment_bridge_notdeleted

    return ret



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.states.makeconf as makeconf

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MakeconfTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.makeconf
    """

    def setup_loader_modules(self):
        return {makeconf: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verify that the variable is in the ``make.conf``
        and has the provided settings.
        """
        name = "makeopts"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_t = MagicMock(return_value=True)
        with patch.dict(makeconf.__salt__, {"makeconf.get_var": mock_t}):
            comt = "Variable {0} is already present in make.conf".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(makeconf.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to verify that the variable is not in the ``make.conf``.
        """
        name = "makeopts"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(return_value=None)
        with patch.dict(makeconf.__salt__, {"makeconf.get_var": mock}):
            comt = "Variable {0} is already absent from make.conf".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(makeconf.absent(name), ret)



# -*- coding: utf-8 -*-
"""
Test case for the vault utils module
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import json
import logging
import os
from copy import copy

# Import Salt libs
import salt.utils.vault as vault
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import ANY, MagicMock, Mock, mock_open, patch

# Import Salt Testing libs
from tests.support.unit import TestCase


class RequestMock(Mock):
    """
    Request Mock
    """

    def get(self, *args, **kwargs):
        return {}


class TestVaultUtils(LoaderModuleMockMixin, TestCase):
    """
    Test case for the vault utils module
    """

    json_success = {
        "request_id": "35df4df1-c3d8-b270-0682-ddb0160c7450",
        "lease_id": "",
        "renewable": False,
        "lease_duration": 0,
        "data": {
            "data": {"something": "myvalue"},
            "metadata": {
                "created_time": "2020-05-02T07:26:12.180848003Z",
                "deletion_time": "",
                "destroyed": False,
                "version": 1,
            },
        },
        "wrap_info": None,
        "warnings": None,
        "auth": None,
    }
    json_denied = {"errors": ["permission denied"]}
    cache_single = {
        "url": "http://127.0.0.1:8200",
        "token": "test",
        "verify": None,
        "uses": 1,
        "lease_duration": 100,
        "issued": 3000,
    }
    cache_uses = {
        "url": "http://127.0.0.1:8200",
        "token": "test",
        "verify": None,
        "uses": 10,
        "lease_duration": 100,
        "issued": 3000,
        "unlimited_use_token": False,
    }
    cache_uses_last = {
        "url": "http://127.0.0.1:8200",
        "token": "test",
        "verify": None,
        "uses": 1,
        "lease_duration": 100,
        "issued": 3000,
        "unlimited_use_token": False,
    }
    cache_unlimited = {
        "url": "http://127.0.0.1:8200",
        "token": "test",
        "verify": None,
        "uses": 0,
        "lease_duration": 100,
        "issued": 3000,
        "unlimited_use_token": True,
    }
    metadata_v2 = {
        "accessor": "kv_f8731f1b",
        "config": {
            "default_lease_ttl": 0,
            "force_no_cache": False,
            "max_lease_ttl": 0,
        },
        "description": "key/value secret storage",
        "external_entropy_access": False,
        "local": False,
        "options": {"version": "2"},
        "path": "secret/",
        "seal_wrap": False,
        "type": "kv",
        "uuid": "1d9431ac-060a-9b63-4572-3ca7ffd78347",
    }
    cache_secret_meta = {"vault_secret_path_metadata": {"secret/mything": metadata_v2}}

    def setup_loader_modules(self):
        return {
            vault: {
                "__opts__": {
                    "vault": {
                        "url": "http://127.0.0.1",
                        "auth": {
                            "token": "test",
                            "method": "token",
                            "uses": 15,
                            "ttl": 500,
                        },
                    },
                    "file_client": "local",
                    "cachedir": "somepath",
                },
                "__grains__": {"id": "test-minion"},
                "__context__": {},
            }
        }

    def _mock_json_response(self, data, status_code=200, reason=""):
        """
        Mock helper for http response
        """
        response = MagicMock()
        response.json = MagicMock(return_value=data)
        response.status_code = status_code
        response.reason = reason
        if status_code == 200:
            response.ok = True
        else:
            response.ok = False
        return Mock(return_value=response)

    def test_make_request_single_use_token_run_ok(self):
        """
        Given single use token in __context__, function should run successful secret lookup with no other modifications
        """
        mock = self._mock_json_response(self.json_success)
        supplied_context = {"vault_token": copy(self.cache_single)}
        expected_headers = {"X-Vault-Token": "test", "Content-Type": "application/json"}
        with patch.dict(vault.__context__, supplied_context):
            with patch("requests.request", mock):
                vault_return = vault.make_request("/secret/my/secret", "key")
                self.assertEqual(vault.__context__, {})
                mock.assert_called_with(
                    "/secret/my/secret",
                    "http://127.0.0.1:8200/key",
                    headers=expected_headers,
                    verify=ANY,
                )
                self.assertEqual(vault_return.json(), self.json_success)

    def test_make_request_single_use_token_run_auth_error(self):
        """
        Given single use token in __context__ and login error, function should request token and re-run
        """
        # Disable logging because simulated http failures are logged as errors
        logging.disable(logging.CRITICAL)
        mock = self._mock_json_response(self.json_denied, status_code=400)
        supplied_context = {"vault_token": copy(self.cache_single)}
        expected_headers = {"X-Vault-Token": "test", "Content-Type": "application/json"}
        with patch.dict(vault.__context__, supplied_context):
            with patch("requests.request", mock):
                with patch.object(vault, "del_cache") as mock_del_cache:
                    vault_return = vault.make_request("/secret/my/secret", "key")
                    self.assertEqual(vault.__context__, {})
                    mock.assert_called_with(
                        "/secret/my/secret",
                        "http://127.0.0.1:8200/key",
                        headers=expected_headers,
                        verify=ANY,
                    )
                    self.assertEqual(vault_return.json(), self.json_denied)
                    mock_del_cache.assert_called()
                    self.assertEqual(mock.call_count, 2)
        logging.disable(logging.NOTSET)

    def test_multi_use_token_successful_run(self):
        """
        Given multi-use token, function should get secret and decrement token
        """
        expected_cache_write = {
            "url": "http://127.0.0.1:8200",
            "token": "test",
            "verify": None,
            "uses": 9,
            "lease_duration": 100,
            "issued": 3000,
            "unlimited_use_token": False,
        }
        mock = self._mock_json_response(self.json_success)
        expected_headers = {"X-Vault-Token": "test", "Content-Type": "application/json"}
        with patch.object(vault, "get_cache") as mock_get_cache:
            mock_get_cache.return_value = copy(self.cache_uses)
            with patch("requests.request", mock):
                with patch.object(vault, "del_cache") as mock_del_cache:
                    with patch.object(vault, "write_cache") as mock_write_cache:
                        vault_return = vault.make_request("/secret/my/secret", "key")
                        mock.assert_called_with(
                            "/secret/my/secret",
                            "http://127.0.0.1:8200/key",
                            headers=expected_headers,
                            verify=ANY,
                        )
                        mock_write_cache.assert_called_with(expected_cache_write)
                        self.assertEqual(vault_return.json(), self.json_success)
                        self.assertEqual(mock.call_count, 1)

    def test_multi_use_token_last_use(self):
        """
        Given last use of multi-use token, function should succeed and flush token cache
        """
        mock = self._mock_json_response(self.json_success)
        expected_headers = {"X-Vault-Token": "test", "Content-Type": "application/json"}
        with patch.object(vault, "get_cache") as mock_get_cache:
            mock_get_cache.return_value = self.cache_uses_last
            with patch("requests.request", mock):
                with patch.object(vault, "del_cache") as mock_del_cache:
                    with patch.object(vault, "write_cache") as mock_write_cache:
                        vault_return = vault.make_request("/secret/my/secret", "key")
                        mock.assert_called_with(
                            "/secret/my/secret",
                            "http://127.0.0.1:8200/key",
                            headers=expected_headers,
                            verify=ANY,
                        )
                        mock_del_cache.assert_called()
                        self.assertEqual(vault_return.json(), self.json_success)
                        self.assertEqual(mock.call_count, 1)

    def test_unlimited_use_token_no_decrement(self):
        """
        Given unlimited-use token, function should succeed not del cache or decrement
        """
        mock = self._mock_json_response(self.json_success)
        expected_headers = {"X-Vault-Token": "test", "Content-Type": "application/json"}
        with patch.object(vault, "get_cache") as mock_get_cache:
            mock_get_cache.return_value = self.cache_unlimited
            with patch("requests.request", mock):
                with patch.object(vault, "del_cache") as mock_del_cache:
                    with patch.object(vault, "write_cache") as mock_write_cache:
                        vault_return = vault.make_request("/secret/my/secret", "key")
                        mock.assert_called_with(
                            "/secret/my/secret",
                            "http://127.0.0.1:8200/key",
                            headers=expected_headers,
                            verify=ANY,
                        )
                        assert (
                            not mock_del_cache.called
                        ), "del cache should not be called for unlimited use token"
                        assert (
                            not mock_write_cache.called
                        ), "write cache should not be called for unlimited use token"
                        self.assertEqual(vault_return.json(), self.json_success)
                        self.assertEqual(mock.call_count, 1)

    def test_get_cache_standard(self):
        """
        test standard first run of no cache file. Should generate new connection and write cache
        """
        with patch.object(vault, "_read_cache_file") as mock_read_cache:
            mock_read_cache.return_value = {}
            with patch.object(
                vault, "get_vault_connection"
            ) as mock_get_vault_connection:
                mock_get_vault_connection.return_value = copy(self.cache_single)
                with patch.object(vault, "write_cache") as mock_write_cache:
                    cache_result = vault.get_cache()
                    mock_write_cache.assert_called_with(copy(self.cache_single))

    def test_get_cache_existing_cache_valid(self):
        """
        test standard valid cache file
        """
        with patch("time.time", return_value=1234):
            with patch.object(vault, "_read_cache_file") as mock_read_cache:
                mock_read_cache.return_value = self.cache_uses
                with patch.object(vault, "write_cache") as mock_write_cache:
                    with patch.object(vault, "del_cache") as mock_del_cache:
                        cache_result = vault.get_cache()
                        assert not mock_write_cache.called
                        assert not mock_del_cache.called
                        self.assertEqual(cache_result, self.cache_uses)

    def test_get_cache_existing_cache_old(self):
        """
        test old cache file
        """
        with patch("time.time", return_value=3101):
            with patch.object(
                vault, "get_vault_connection"
            ) as mock_get_vault_connection:
                mock_get_vault_connection.return_value = self.cache_uses
                with patch.object(vault, "_read_cache_file") as mock_read_cache:
                    mock_read_cache.return_value = self.cache_uses
                    with patch.object(vault, "write_cache") as mock_write_cache:
                        with patch.object(vault, "del_cache") as mock_del_cache:
                            cache_result = vault.get_cache()
                            assert mock_del_cache.called
                            assert mock_write_cache.called
                            self.assertEqual(cache_result, self.cache_uses)

    def test_write_cache_standard(self):
        """
        Test write cache with standard single use token
        """
        function_response = vault.write_cache(copy(self.cache_single))
        self.assertEqual(vault.__context__["vault_token"], copy(self.cache_single))
        self.assertTrue(function_response)

    def test_write_cache_multi_use_token(self):
        """
        Test write cache with multi-use token
        """
        expected_write = {
            "url": "http://127.0.0.1:8200",
            "token": "test",
            "verify": None,
            "uses": 10,
            "lease_duration": 100,
            "issued": 3000,
            "unlimited_use_token": False,
        }
        with patch("salt.utils.files.fpopen", mock_open()) as mock_fpopen:
            function_response = vault.write_cache(self.cache_uses)
            assert mock_fpopen.call_count == 1
            self.assertListEqual(
                list(mock_fpopen.filehandles),
                [os.path.join("somepath", "salt_vault_token")],
            )
            opens = mock_fpopen.filehandles[
                os.path.join("somepath", "salt_vault_token")
            ]
            write_calls_output = json.loads(opens[0].write_calls[0])
            self.assertDictEqual(write_calls_output, expected_write)
            self.assertTrue(function_response)

    def test_write_cache_unlimited_token(self):
        """
        Test write cache with unlimited use token
        """
        write_data = {
            "url": "http://127.0.0.1:8200",
            "token": "test",
            "verify": None,
            "uses": 0,
            "lease_duration": 100,
            "issued": 3000,
        }
        expected_write = {
            "url": "http://127.0.0.1:8200",
            "token": "test",
            "verify": None,
            "uses": 0,
            "lease_duration": 100,
            "issued": 3000,
            "unlimited_use_token": True,
        }

        with patch("salt.utils.files.fpopen", mock_open()) as mock_fpopen:
            function_response = vault.write_cache(write_data)
            assert mock_fpopen.call_count == 1
            self.assertListEqual(
                list(mock_fpopen.filehandles),
                [os.path.join("somepath", "salt_vault_token")],
            )
            opens = mock_fpopen.filehandles[
                os.path.join("somepath", "salt_vault_token")
            ]
            write_calls_output = json.loads(opens[0].write_calls[0])
            self.assertEqual(write_calls_output, expected_write)
            self.assertTrue(function_response)

    def test_path_is_v2(self):
        """
        Validated v2 path is detected as vault kv v2
        """
        expected_return = {
            "v2": True,
            "data": "secret/data/mything",
            "metadata": "secret/metadata/mything",
            "delete": "secret/mything",
            "type": "kv",
            "destroy": "secret/destroy/mything",
        }
        with patch.object(vault, "_get_secret_path_metadata") as mock_get_metadata:
            mock_get_metadata.return_value = self.metadata_v2
            function_return = vault.is_v2("secret/mything")
            self.assertEqual(function_return, expected_return)

    def test_get_secret_path_metadata_no_cache(self):
        """
        test with no cache file
        """
        make_request_response = {
            "request_id": "b82f2df7-a9b6-920c-0ed2-a3463b996f9e",
            "lease_id": "",
            "renewable": False,
            "lease_duration": 0,
            "data": self.metadata_v2,
            "wrap_info": None,
            "warnings": None,
            "auth": None,
        }
        cache_object = copy(self.cache_uses)
        expected_cache_object = copy(self.cache_uses)
        expected_cache_object.update(copy(self.cache_secret_meta))
        secret_path = "secret/mything"
        mock = self._mock_json_response(make_request_response)
        with patch.object(vault, "_read_cache_file") as mock_read_cache:
            mock_read_cache.return_value = cache_object
            with patch.object(vault, "write_cache") as mock_write_cache:
                with patch("salt.utils.vault.make_request", mock):
                    function_result = vault._get_secret_path_metadata(secret_path)
                    self.assertEqual(function_result, self.metadata_v2)
                    mock_write_cache.assert_called_with(cache_object)
                    self.assertEqual(cache_object, expected_cache_object)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import textwrap

# Import Salt Libs
import salt.config
import salt.loader
import salt.states.boto_sqs as boto_sqs

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoSqsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_sqs
    """

    def setup_loader_modules(self):
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto3", "yaml", "args", "systemd", "path", "platform"],
            context={},
        )
        return {boto_sqs: {"__utils__": utils}}

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the SQS queue exists.
        """
        name = "mysqs"
        attributes = {"DelaySeconds": 20}
        base_ret = {"name": name, "changes": {}}

        mock = MagicMock(
            side_effect=[{"result": b} for b in [False, False, True, True]],
        )
        mock_bool = MagicMock(return_value={"error": "create error"})
        mock_attr = MagicMock(return_value={"result": {}})
        with patch.dict(
            boto_sqs.__salt__,
            {
                "boto_sqs.exists": mock,
                "boto_sqs.create": mock_bool,
                "boto_sqs.get_attributes": mock_attr,
            },
        ):
            with patch.dict(boto_sqs.__opts__, {"test": False}):
                comt = ["Failed to create SQS queue {0}: create error".format(name,)]
                ret = base_ret.copy()
                ret.update({"result": False, "comment": comt})
                self.assertDictEqual(boto_sqs.present(name), ret)

            with patch.dict(boto_sqs.__opts__, {"test": True}):
                comt = ["SQS queue {0} is set to be created.".format(name)]
                ret = base_ret.copy()
                ret.update(
                    {
                        "result": None,
                        "comment": comt,
                        "changes": {"old": None, "new": "mysqs"},
                    }
                )
                self.assertDictEqual(boto_sqs.present(name), ret)
                diff = textwrap.dedent(
                    """\
                    ---
                    +++
                    @@ -1 +1 @@
                    -{}
                    +DelaySeconds: 20

                """
                ).splitlines()
                # Difflib adds a trailing space after the +++/--- lines,
                # programatically add them back here. Having them in the test
                # file itself is not feasible since a few popular plugins for
                # vim will remove trailing whitespace.
                for idx in (0, 1):
                    diff[idx] += " "
                diff = "\n".join(diff)

                comt = [
                    "SQS queue mysqs present.",
                    "Attribute(s) DelaySeconds set to be updated:\n{0}".format(diff,),
                ]
                ret.update({"comment": comt, "changes": {"attributes": {"diff": diff}}})
                self.assertDictEqual(boto_sqs.present(name, attributes), ret)

            comt = ["SQS queue mysqs present."]
            ret = base_ret.copy()
            ret.update({"result": True, "comment": comt})
            self.assertDictEqual(boto_sqs.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named sqs queue is deleted.
        """
        name = "test.example.com."
        base_ret = {"name": name, "changes": {}}

        mock = MagicMock(side_effect=[{"result": False}, {"result": True}])
        with patch.dict(boto_sqs.__salt__, {"boto_sqs.exists": mock}):
            comt = "SQS queue {0} does not exist in None.".format(name)
            ret = base_ret.copy()
            ret.update({"result": True, "comment": comt})
            self.assertDictEqual(boto_sqs.absent(name), ret)

            with patch.dict(boto_sqs.__opts__, {"test": True}):
                comt = "SQS queue {0} is set to be removed.".format(name)
                ret = base_ret.copy()
                ret.update(
                    {
                        "result": None,
                        "comment": comt,
                        "changes": {"old": name, "new": None},
                    }
                )
                self.assertDictEqual(boto_sqs.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.postgres_schema as postgres_schema

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresSchemaTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_schema
    """

    def setup_loader_modules(self):
        return {postgres_schema: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named schema is present in the database.
        """
        name = "myname"
        dbname = "mydb"

        ret = {
            "name": name,
            "dbname": dbname,
            "changes": {},
            "result": True,
            "comment": "",
        }

        mock = MagicMock(return_value=name)
        with patch.dict(postgres_schema.__salt__, {"postgres.schema_get": mock}):
            with patch.dict(postgres_schema.__opts__, {"test": False}):
                comt = "Schema {0} already exists in database {1}".format(name, dbname)
                ret.update({"comment": comt})
                self.assertDictEqual(postgres_schema.present(dbname, name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named schema is absent.
        """
        name = "myname"
        dbname = "mydb"

        ret = {
            "name": name,
            "dbname": dbname,
            "changes": {},
            "result": True,
            "comment": "",
        }

        mock_t = MagicMock(side_effect=[True, False])
        mock = MagicMock(side_effect=[True, True, True, False])
        with patch.dict(
            postgres_schema.__salt__,
            {"postgres.schema_exists": mock, "postgres.schema_remove": mock_t},
        ):
            with patch.dict(postgres_schema.__opts__, {"test": True}):
                comt = "Schema {0} is set to be removed from database {1}".format(
                    name, dbname
                )
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(postgres_schema.absent(dbname, name), ret)

            with patch.dict(postgres_schema.__opts__, {"test": False}):
                comt = "Schema {0} has been removed from database {1}".format(
                    name, dbname
                )
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Absent"}}
                )
                self.assertDictEqual(postgres_schema.absent(dbname, name), ret)

                comt = "Schema {0} failed to be removed".format(name)
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(postgres_schema.absent(dbname, name), ret)

            comt = (
                "Schema {0} is not present in database {1},"
                " so it cannot be removed".format(name, dbname)
            )
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(postgres_schema.absent(dbname, name), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.mysql_query as mysql_query

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MysqlQueryTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.mysql_query
    """

    def setup_loader_modules(self):
        return {mysql_query: {}}

    # 'run' function tests: 1

    def test_run(self):
        """
        Test to execute an arbitrary query on the specified database.
        """
        name = "query_id"
        database = "my_database"
        query = "SELECT * FROM table;"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        mock_str = MagicMock(return_value="salt")
        mock_none = MagicMock(return_value=None)
        mock_dict = MagicMock(return_value={"salt": "SALT"})
        mock_lst = MagicMock(return_value=["grain"])
        with patch.dict(mysql_query.__salt__, {"mysql.db_exists": mock_f}):
            with patch.object(mysql_query, "_get_mysql_error", mock_str):
                ret.update({"comment": "salt", "result": False})
                self.assertDictEqual(mysql_query.run(name, database, query), ret)

            with patch.object(mysql_query, "_get_mysql_error", mock_none):
                comt = "Database {0} is not present".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(mysql_query.run(name, database, query), ret)

        with patch.dict(
            mysql_query.__salt__,
            {
                "mysql.db_exists": mock_t,
                "grains.ls": mock_lst,
                "grains.get": mock_dict,
                "mysql.query": mock_str,
            },
        ):
            comt = "No execution needed. Grain grain already set"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(
                mysql_query.run(
                    name,
                    database,
                    query,
                    output="grain",
                    grain="grain",
                    overwrite=False,
                ),
                ret,
            )

            with patch.dict(mysql_query.__opts__, {"test": True}):
                comt = "Query would execute, storing result in grain: grain"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    mysql_query.run(
                        name, database, query, output="grain", grain="grain"
                    ),
                    ret,
                )

                comt = "Query would execute, storing result" " in grain: grain:salt"
                ret.update({"comment": comt})
                self.assertDictEqual(
                    mysql_query.run(
                        name, database, query, output="grain", grain="grain", key="salt"
                    ),
                    ret,
                )

                comt = "Query would execute, storing result in file: salt"
                ret.update({"comment": comt})
                self.assertDictEqual(
                    mysql_query.run(
                        name, database, query, output="salt", grain="grain"
                    ),
                    ret,
                )

                comt = "Query would execute, not storing result"
                ret.update({"comment": comt})
                self.assertDictEqual(mysql_query.run(name, database, query), ret)

            comt = "No execution needed. Grain grain:salt already set"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(
                mysql_query.run(
                    name,
                    database,
                    query,
                    output="grain",
                    grain="grain",
                    key="salt",
                    overwrite=False,
                ),
                ret,
            )

            comt = "Error: output type 'grain' needs the grain parameter\n"
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(
                mysql_query.run(name, database, query, output="grain"), ret
            )

            with patch.object(os.path, "isfile", mock_t):
                comt = "No execution needed. File salt already set"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(
                    mysql_query.run(
                        name,
                        database,
                        query,
                        output="salt",
                        grain="grain",
                        overwrite=False,
                    ),
                    ret,
                )

            with patch.dict(mysql_query.__opts__, {"test": False}):
                ret.update({"comment": "salt", "changes": {"query": "Executed"}})
                self.assertDictEqual(mysql_query.run(name, database, query), ret)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import Salt Libs
import salt.states.blockdev as blockdev
import salt.utils.path

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch
from tests.support.unit import TestCase


class BlockdevTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.blockdev
    """

    def setup_loader_modules(self):
        return {blockdev: {}}

    # 'tuned' function tests: 1

    def test_tuned(self):
        """
        Test to manage options of block device
        """
        name = "/dev/vg/master-data"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        comt = ("Changes to {0} cannot be applied. " "Not a block device. ").format(
            name
        )
        with patch.dict(blockdev.__salt__, {"file.is_blkdev": False}):
            ret.update({"comment": comt})
            self.assertDictEqual(blockdev.tuned(name), ret)

        comt = "Changes to {0} will be applied ".format(name)
        with patch.dict(blockdev.__salt__, {"file.is_blkdev": True}):
            ret.update({"comment": comt, "result": None})
            with patch.dict(blockdev.__opts__, {"test": True}):
                self.assertDictEqual(blockdev.tuned(name), ret)

    # 'formatted' function tests: 1

    def test_formatted(self):
        """
        Test to manage filesystems of partitions.
        """
        name = "/dev/vg/master-data"

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        with patch.object(
            os.path, "exists", MagicMock(side_effect=[False, True, True, True, True])
        ):
            comt = "{0} does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(blockdev.formatted(name), ret)

            mock_ext4 = MagicMock(return_value="ext4")

            # Test state return when block device is already in the correct state
            with patch.dict(blockdev.__salt__, {"cmd.run": mock_ext4}):
                comt = "{0} already formatted with ext4".format(name)
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(blockdev.formatted(name), ret)

            # Test state return when provided block device is an invalid fs_type
            with patch.dict(blockdev.__salt__, {"cmd.run": MagicMock(return_value="")}):
                ret.update({"comment": "Invalid fs_type: foo-bar", "result": False})
                with patch.object(
                    salt.utils.path, "which", MagicMock(return_value=False)
                ):
                    self.assertDictEqual(
                        blockdev.formatted(name, fs_type="foo-bar"), ret
                    )

            # Test state return when provided block device state will change and test=True
            with patch.dict(
                blockdev.__salt__, {"cmd.run": MagicMock(return_value="new-thing")}
            ):
                comt = "Changes to {0} will be applied ".format(name)
                ret.update({"comment": comt, "result": None})
                with patch.object(
                    salt.utils.path, "which", MagicMock(return_value=True)
                ):
                    with patch.dict(blockdev.__opts__, {"test": True}):
                        self.assertDictEqual(blockdev.formatted(name), ret)

            # Test state return when block device format fails
            with patch.dict(
                blockdev.__salt__,
                {
                    "cmd.run": MagicMock(return_value=mock_ext4),
                    "disk.format": MagicMock(return_value=True),
                },
            ):
                comt = "Failed to format {0}".format(name)
                ret.update({"comment": comt, "result": False})
                with patch.object(
                    salt.utils.path, "which", MagicMock(return_value=True)
                ):
                    with patch.dict(blockdev.__opts__, {"test": False}):
                        self.assertDictEqual(blockdev.formatted(name), ret)

    def test__checkblk(self):
        """
        Confirm that we call cmd.run with ignore_retcode=True
        """
        cmd_mock = Mock()
        with patch.dict(blockdev.__salt__, {"cmd.run": cmd_mock}):
            blockdev._checkblk("/dev/foo")

        cmd_mock.assert_called_once_with(
            ["blkid", "-o", "value", "-s", "TYPE", "/dev/foo"], ignore_retcode=True
        )



# -*- coding: utf-8 -*-
"""
Test utility methods that communicate with SMB shares.
"""
from __future__ import absolute_import

import getpass
import logging
import os
import signal
import subprocess
import tempfile
import time

import salt.utils.files
import salt.utils.path
import salt.utils.smb
from tests.support.case import TestCase
from tests.support.unit import skipIf

log = logging.getLogger(__name__)
CONFIG = (
    "[global]\n"
    "realm = saltstack.com\n"
    "interfaces = lo 127.0.0.0/8\n"
    "smb ports = 1445\n"
    "log level = 2\n"
    "map to guest = Bad User\n"
    "enable core files = no\n"
    "passdb backend = smbpasswd\n"
    "smb passwd file = {passwdb}\n"
    "lock directory = {samba_dir}\n"
    "state directory = {samba_dir}\n"
    "cache directory = {samba_dir}\n"
    "pid directory = {samba_dir}\n"
    "private dir = {samba_dir}\n"
    "ncalrpc dir = {samba_dir}\n"
    "socket options = IPTOS_LOWDELAY TCP_NODELAY\n"
    "min receivefile size = 0\n"
    "write cache size = 0\n"
    "client ntlmv2 auth = no\n"
    "client min protocol = SMB3_11\n"
    "client plaintext auth = no\n"
    "\n"
    "[public]\n"
    "path = {public_dir}\n"
    "read only = no\n"
    "guest ok = no\n"
    "writeable = yes\n"
    "force user = {user}\n"
)
TBE = (
    "{}:0:XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX:AC8E657F8"
    "3DF82BEEA5D43BDAF7800CC:[U          ]:LCT-507C14C7:"
)


def which_smbd():
    """
    Find the smbd executable and cache the result if it exits.
    """
    if hasattr(which_smbd, "cached_result"):
        return which_smbd.cached_result
    smbd = salt.utils.path.which("smbd")
    if smbd:
        which_smbd.cached_result = smbd
    return smbd


@skipIf(not which_smbd(), reason="smbd binary not found")
@skipIf(
    not salt.utils.smb.HAS_SMBPROTOCOL, '"smbprotocol" needs to be installed.',
)
class TestSmb(TestCase):

    _smbd = None

    @staticmethod
    def check_pid(pid):
        try:
            os.kill(pid, 0)
        except OSError:
            return False
        else:
            return True

    @classmethod
    def setUpClass(cls):
        tmpdir = tempfile.mkdtemp()
        cls.samba_dir = os.path.join(tmpdir, "samba")
        cls.public_dir = os.path.join(tmpdir, "public")
        os.makedirs(cls.samba_dir)
        os.makedirs(cls.public_dir)
        os.chmod(cls.samba_dir, 0o775)
        os.chmod(cls.public_dir, 0o775)
        passwdb = os.path.join(tmpdir, "passwdb")
        cls.username = getpass.getuser()
        with salt.utils.files.fopen(passwdb, "w") as fp:
            fp.write(TBE.format(cls.username))
        samba_conf = os.path.join(tmpdir, "smb.conf")
        with salt.utils.files.fopen(samba_conf, "w") as fp:
            fp.write(
                CONFIG.format(
                    samba_dir=cls.samba_dir,
                    public_dir=cls.public_dir,
                    passwdb=passwdb,
                    user=cls.username,
                )
            )
        cls._smbd = subprocess.Popen(
            "{0} -FS -P0 -s {1}".format(which_smbd(), samba_conf), shell=True
        )
        time.sleep(1)
        pidfile = os.path.join(cls.samba_dir, "smbd.pid")
        with salt.utils.files.fopen(pidfile, "r") as fp:
            cls._pid = int(fp.read().strip())
        if not cls.check_pid(cls._pid):
            raise Exception("Unable to locate smbd's pid file")

    @classmethod
    def tearDownClass(cls):
        log.warning("teardown")
        os.kill(cls._pid, signal.SIGTERM)

    def test_write_file(self):
        """
        Transfer a file over SMB
        """
        name = "test_write_file.txt"
        content = "write test file content"
        share_path = os.path.join(self.public_dir, name)
        assert not os.path.exists(share_path)

        local_path = tempfile.mktemp()
        with salt.utils.files.fopen(local_path, "w") as fp:
            fp.write(content)
        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.put_file(local_path, name, "public", conn=conn)
        conn.close()

        assert os.path.exists(share_path)
        with salt.utils.files.fopen(share_path, "r") as fp:
            result = fp.read()
        assert result == content

    def test_write_str(self):
        """
        Write a string to a file over SMB
        """
        name = "test_write_str.txt"
        content = "write test file content"
        share_path = os.path.join(self.public_dir, name)
        assert not os.path.exists(share_path)
        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.put_str(content, name, "public", conn=conn)
        conn.close()

        assert os.path.exists(share_path)
        with salt.utils.files.fopen(share_path, "r") as fp:
            result = fp.read()
        assert result == content

    def test_delete_file(self):
        """
        Validate deletion of files over SMB
        """
        name = "test_delete_file.txt"
        content = "read test file content"
        share_path = os.path.join(self.public_dir, name)
        with salt.utils.files.fopen(share_path, "w") as fp:
            fp.write(content)
        assert os.path.exists(share_path)

        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.delete_file(name, "public", conn=conn)
        conn.close()

        assert not os.path.exists(share_path)

    def test_mkdirs(self):
        """
        Create directories over SMB
        """
        dir_name = "mkdirs/test"
        share_path = os.path.join(self.public_dir, dir_name)
        assert not os.path.exists(share_path)

        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.mkdirs(dir_name, "public", conn=conn)
        conn.close()

        assert os.path.exists(share_path)

    def test_delete_dirs(self):
        """
        Validate deletion of directoreies over SMB
        """
        dir_name = "deldirs"
        subdir_name = "deldirs/test"
        local_path = os.path.join(self.public_dir, subdir_name)
        os.makedirs(local_path)
        assert os.path.exists(local_path)

        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.delete_directory(subdir_name, "public", conn=conn)
        conn.close()

        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        salt.utils.smb.delete_directory(dir_name, "public", conn=conn)
        conn.close()

        assert not os.path.exists(local_path)
        assert not os.path.exists(os.path.join(self.public_dir, dir_name))

    def test_connection(self):
        """
        Validate creation of an SMB connection
        """
        conn = salt.utils.smb.get_conn("127.0.0.1", self.username, "foo", port=1445)
        conn.close()



# -*- coding: utf-8 -*-
"""
This is the default glob matcher function.
"""
from __future__ import absolute_import, print_function, unicode_literals

import fnmatch

from salt.ext import six  # pylint: disable=3rd-party-module-not-gated


def match(tgt, opts=None):
    """
    Returns true if the passed glob matches the id
    """
    if not opts:
        opts = __opts__
    minion_id = opts.get("minion_id", opts["id"])
    if not isinstance(tgt, six.string_types):
        return False

    return fnmatch.fnmatch(minion_id, tgt)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.ssh_auth as ssh_auth

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SshAuthTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.ssh_auth
    """

    def setup_loader_modules(self):
        return {ssh_auth: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to verifies that the specified SSH key
        is present for the specified user.
        """
        name = "sshkeys"
        user = "root"
        source = "salt://ssh_keys/id_rsa.pub"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock = MagicMock(return_value="exists")
        mock_data = MagicMock(side_effect=["replace", "new"])
        with patch.dict(
            ssh_auth.__salt__, {"ssh.check_key": mock, "ssh.set_auth_key": mock_data}
        ):
            with patch.dict(ssh_auth.__opts__, {"test": True}):
                comt = (
                    "The authorized host key sshkeys is already "
                    "present for user root"
                )
                ret.update({"comment": comt})
                self.assertDictEqual(ssh_auth.present(name, user, source), ret)

            with patch.dict(ssh_auth.__opts__, {"test": False}):
                comt = "The authorized host key sshkeys " "for user root was updated"
                ret.update({"comment": comt, "changes": {name: "Updated"}})
                self.assertDictEqual(ssh_auth.present(name, user, source), ret)

                comt = "The authorized host key sshkeys " "for user root was added"
                ret.update({"comment": comt, "changes": {name: "New"}})
                self.assertDictEqual(ssh_auth.present(name, user, source), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to verifies that the specified SSH key is absent.
        """
        name = "sshkeys"
        user = "root"
        source = "salt://ssh_keys/id_rsa.pub"

        ret = {"name": name, "changes": {}, "result": None, "comment": ""}

        mock = MagicMock(
            side_effect=["User authorized keys file not present", "Key removed"]
        )
        mock_up = MagicMock(side_effect=["update", "updated"])
        with patch.dict(
            ssh_auth.__salt__, {"ssh.rm_auth_key": mock, "ssh.check_key": mock_up}
        ):
            with patch.dict(ssh_auth.__opts__, {"test": True}):
                comt = "Key sshkeys for user root is set for removal"
                ret.update({"comment": comt})
                self.assertDictEqual(ssh_auth.absent(name, user, source), ret)

                comt = "Key is already absent"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(ssh_auth.absent(name, user, source), ret)

            with patch.dict(ssh_auth.__opts__, {"test": False}):
                comt = "User authorized keys file not present"
                ret.update({"comment": comt, "result": False})
                self.assertDictEqual(ssh_auth.absent(name, user, source), ret)

                comt = "Key removed"
                ret.update(
                    {"comment": comt, "result": True, "changes": {name: "Removed"}}
                )
                self.assertDictEqual(ssh_auth.absent(name, user, source), ret)

    def test_manage(self):
        """
        Test to verifies that the specified SSH key is absent.
        """
        user = "root"
        ret = {"name": "", "changes": {}, "result": None, "comment": ""}

        mock_rm = MagicMock(
            side_effect=["User authorized keys file not present", "Key removed"]
        )
        mock_up = MagicMock(side_effect=["update", "updated"])
        mock_set = MagicMock(side_effect=["replace", "new"])
        mock_keys = MagicMock(
            return_value={
                "somekey": {
                    "enc": "ssh-rsa",
                    "comment": "user@host",
                    "options": [],
                    "fingerprint": "b7",
                }
            }
        )
        with patch.dict(
            ssh_auth.__salt__,
            {
                "ssh.rm_auth_key": mock_rm,
                "ssh.set_auth_key": mock_set,
                "ssh.check_key": mock_up,
                "ssh.auth_keys": mock_keys,
            },
        ):
            with patch("salt.states.ssh_auth.present") as call_mocked_present:
                mock_present = {"comment": "", "changes": {}, "result": None}
                call_mocked_present.return_value = mock_present
                with patch.dict(ssh_auth.__opts__, {"test": True}):
                    # test: expected keys found. No chanages
                    self.assertDictEqual(
                        ssh_auth.manage("sshid", ["somekey"], user), ret
                    )

                    comt = "somekey Key set for removal"
                    ret.update({"comment": comt})
                    # test: unexpected sshkey found. Should be removed.
                    self.assertDictEqual(ssh_auth.manage("sshid", [], user), ret)

            with patch("salt.states.ssh_auth.present") as call_mocked_present:
                mock_present = {"comment": "", "changes": {}, "result": True}
                call_mocked_present.return_value = mock_present
                with patch.dict(ssh_auth.__opts__, {"test": False}):
                    # expected keys found. No changes
                    ret = {"name": "", "changes": {}, "result": True, "comment": ""}
                    self.assertDictEqual(
                        ssh_auth.manage("sshid", ["somekey"], user), ret
                    )

                    with patch("salt.states.ssh_auth.absent") as call_mocked_absent:
                        mock_absent = {"comment": "Key removed"}
                        call_mocked_absent.return_value = mock_absent
                        ret.update(
                            {
                                "comment": "",
                                "result": True,
                                "changes": {"somekey": "Key removed"},
                            }
                        )
                        # unexpected sshkey found. Was removed.
                        self.assertDictEqual(
                            ssh_auth.manage("sshid", ["addkey"], user), ret
                        )

            # add a key
            with patch("salt.states.ssh_auth.present") as call_mocked_present:
                mock_present = {
                    "comment": "The authorized host key newkey for user {} was added".format(
                        user
                    ),
                    "changes": {"newkey": "New"},
                    "result": True,
                }
                call_mocked_present.return_value = mock_present
                with patch.dict(ssh_auth.__opts__, {"test": False}):
                    # added key newkey
                    ret = {
                        "name": "",
                        "changes": {"newkey": "New"},
                        "result": True,
                        "comment": "",
                    }
                    self.assertDictEqual(
                        ssh_auth.manage("sshid", ["newkey", "somekey"], user), ret
                    )



# -*- coding: utf-8 -*-
"""
    :codeauthor: `Anthony Shaw <anthonyshaw@apache.org>`

    tests.unit.cloud.clouds.gce_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import collections

# Import Salt Libs
from salt.cloud.clouds import gce
from salt.exceptions import SaltCloudSystemExit
from salt.utils.versions import LooseVersion

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.mock import __version__ as mock_version
from tests.support.mock import patch
from tests.support.unit import TestCase

try:
    import libcloud.security

    HAS_LIBCLOUD = True
except ImportError:
    HAS_LIBCLOUD = False


VM_NAME = "kings_landing"
DUMMY_TOKEN = {
    "refresh_token": None,
    "client_id": "dany123",
    "client_secret": "lalalalalalala",
    "grant_type": "refresh_token",
}

# Use certifi if installed
try:
    if HAS_LIBCLOUD:
        # This work-around for Issue #32743 is no longer needed for libcloud >=
        # 1.4.0. However, older versions of libcloud must still be supported
        # with this work-around. This work-around can be removed when the
        # required minimum version of libcloud is 2.0.0 (See PR #40837 - which
        # is implemented in Salt 2018.3.0).
        if LooseVersion(libcloud.__version__) < LooseVersion("1.4.0"):
            import certifi

            libcloud.security.CA_CERTS_PATH.append(certifi.where())
except ImportError:
    pass


class DummyGCEConn(object):
    def __init__(self):
        self.create_node = MagicMock()

    def __getattr__(self, attr):
        if attr != "create_node":
            # Return back the first thing passed in (i.e. don't call out to get
            # the override value).
            return lambda *args, **kwargs: args[0]


class GCETestCase(TestCase, LoaderModuleMockMixin):
    """
    Unit TestCase for salt.cloud.clouds.gce module.
    """

    def setup_loader_modules(self):
        return {
            gce: {
                "__active_provider_name__": "",
                "__utils__": {
                    "cloud.fire_event": MagicMock(),
                    "cloud.filter_event": MagicMock(),
                },
                "__opts__": {
                    "sock_dir": True,
                    "transport": True,
                    "providers": {
                        "my-google-cloud": {
                            "gce": {
                                "project": "daenerys-cloud",
                                "service_account_email_address": "dany@targaryen.westeros.cloud",
                                "service_account_private_key": "/home/dany/PRIVKEY.pem",
                                "driver": "gce",
                                "ssh_interface": "public_ips",
                            }
                        }
                    },
                },
            }
        }

    def setUp(self):
        self.location = collections.namedtuple("Location", "name")("chicago")
        self.vm_ = {
            "name": "new",
            "driver": "gce",
            "profile": None,
            "size": 1234,
            "image": "myimage",
            "location": self.location,
            "ex_network": "mynetwork",
            "ex_subnetwork": "mysubnetwork",
            "ex_tags": "mytags",
            "ex_metadata": "metadata",
        }
        self.conn = DummyGCEConn()

    def tearDown(self):
        del self.vm_
        del self.conn

    def test_destroy_call(self):
        """
        Tests that a SaltCloudSystemExit is raised when trying to call destroy
        with --function or -f.
        """
        self.assertRaises(
            SaltCloudSystemExit, gce.destroy, vm_name=VM_NAME, call="function"
        )

    def test_fail_virtual_missing_deps(self):
        # Missing deps
        with patch("salt.config.check_driver_dependencies", return_value=False):
            v = gce.__virtual__()
            self.assertEqual(v, False)

    def test_fail_virtual_deps_missing_config(self):
        with patch("salt.config.check_driver_dependencies", return_value=True):
            with patch("salt.config.is_provider_configured", return_value=False):
                v = gce.__virtual__()
                self.assertEqual(v, False)

    def test_import(self):
        """
        Test that the module picks up installed deps
        """
        with patch("salt.config.check_driver_dependencies", return_value=True) as p:
            get_deps = gce.get_dependencies()
            self.assertEqual(get_deps, True)
            if LooseVersion(mock_version) >= LooseVersion("2.0.0"):
                self.assert_called_once(p)

    def test_provider_matches(self):
        """
        Test that the first configured instance of a gce driver is matched
        """
        p = gce.get_configured_provider()
        self.assertNotEqual(p, None)

    def test_request_instance_with_accelerator(self):
        """
        Test requesting an instance with GCE accelerators
        """

        self.vm_.update({"ex_accelerator_type": "foo", "ex_accelerator_count": 42})
        call_kwargs = {
            "ex_disk_type": "pd-standard",
            "ex_metadata": {"items": [{"value": None, "key": "salt-cloud-profile"}]},
            "ex_accelerator_count": 42,
            "name": "new",
            "ex_service_accounts": None,
            "external_ip": "ephemeral",
            "ex_accelerator_type": "foo",
            "ex_tags": None,
            "ex_disk_auto_delete": True,
            "ex_network": "default",
            "ex_disks_gce_struct": None,
            "ex_preemptible": False,
            "ex_can_ip_forward": False,
            "ex_on_host_maintenance": "TERMINATE",
            "location": self.location,
            "ex_subnetwork": None,
            "image": "myimage",
            "size": 1234,
        }

        with patch(
            "salt.cloud.clouds.gce.get_conn", MagicMock(return_value=self.conn)
        ), patch("salt.cloud.clouds.gce.show_instance", MagicMock()), patch(
            "salt.cloud.clouds.gce.LIBCLOUD_VERSION_INFO", (2, 3, 0)
        ):
            gce.request_instance(self.vm_)
            self.conn.create_node.assert_called_once_with(**call_kwargs)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.influxdb08_database as influxdb08_database

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class InfluxdbDatabaseTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.influxdb08_database
    """

    def setup_loader_modules(self):
        return {influxdb08_database: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named database is present.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[False, False, False, True])
        mock_t = MagicMock(side_effect=[True, False])
        with patch.dict(
            influxdb08_database.__salt__,
            {"influxdb08.db_exists": mock, "influxdb08.db_create": mock_t},
        ):
            with patch.dict(influxdb08_database.__opts__, {"test": True}):
                comt = "Database {0} is absent and needs to be created".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(influxdb08_database.present(name), ret)

            with patch.dict(influxdb08_database.__opts__, {"test": False}):
                comt = "Database {0} has been created".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"salt": "Present"}}
                )
                self.assertDictEqual(influxdb08_database.present(name), ret)

                comt = "Failed to create database {0}".format(name)
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(influxdb08_database.present(name), ret)

            comt = "Database {0} is already present, so cannot be created".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(influxdb08_database.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named database is absent.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[True, True, True, False])
        mock_t = MagicMock(side_effect=[True, False])
        with patch.dict(
            influxdb08_database.__salt__,
            {"influxdb08.db_exists": mock, "influxdb08.db_remove": mock_t},
        ):
            with patch.dict(influxdb08_database.__opts__, {"test": True}):
                comt = "Database {0} is present and needs to be removed".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(influxdb08_database.absent(name), ret)

            with patch.dict(influxdb08_database.__opts__, {"test": False}):
                comt = "Database {0} has been removed".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"salt": "Absent"}}
                )
                self.assertDictEqual(influxdb08_database.absent(name), ret)

                comt = "Failed to remove database {0}".format(name)
                ret.update({"comment": comt, "result": False, "changes": {}})
                self.assertDictEqual(influxdb08_database.absent(name), ret)

            comt = "Database {0} is not present, so it cannot be removed".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(influxdb08_database.absent(name), ret)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import glob
import logging
import os
import shutil
import threading
import time

from salt.ext import six
from salt.ext.six.moves import range
from tests.support.case import SSHCase
from tests.support.helpers import flaky, slowTest
from tests.support.runtests import RUNTIME_VARS

SSH_SLS = "ssh_state_tests"
SSH_SLS_FILE = "/tmp/salt_test_file"

log = logging.getLogger(__name__)


class SSHStateTest(SSHCase):
    """
    testing the state system with salt-ssh
    """

    def _check_dict_ret(self, ret, val, exp_ret, equal=True):
        self.assertIsInstance(ret, dict)
        for key, value in ret.items():
            self.assertIsInstance(value, dict)
            if equal:
                self.assertEqual(value[val], exp_ret)
            else:
                self.assertNotEqual(value[val], exp_ret)

    def _check_request(self, empty=False):
        check = self.run_function("state.check_request", wipe=False)
        if empty:
            self.assertFalse(bool(check), "bool({0}) is not False".format(check))
        else:
            self._check_dict_ret(
                ret=check["default"]["test_run"]["local"]["return"],
                val="__sls__",
                exp_ret=SSH_SLS,
            )

    @slowTest
    def test_state_apply(self):
        """
        test state.apply with salt-ssh
        """
        ret = self.run_function("state.apply", [SSH_SLS])
        self._check_dict_ret(ret=ret, val="__sls__", exp_ret=SSH_SLS)

        check_file = self.run_function("file.file_exists", [SSH_SLS_FILE])
        self.assertTrue(check_file)

    @slowTest
    def test_state_sls_id(self):
        """
        test state.sls_id with salt-ssh
        """
        # check state.sls_id with test=True
        ret = self.run_function("state.sls_id", ["ssh-file-test", SSH_SLS, "test=True"])
        self._check_dict_ret(
            ret=ret,
            val="comment",
            exp_ret=(
                "The file {} is set to be changed\n"
                "Note: No changes made, actual changes may\n"
                "be different due to other states."
            ).format(SSH_SLS_FILE),
        )

        # check state.sls_id without test=True
        ret = self.run_function("state.sls_id", ["ssh-file-test", SSH_SLS])
        self._check_dict_ret(ret=ret, val="__sls__", exp_ret=SSH_SLS)

        # make sure the other id in the state was not run
        self._check_dict_ret(ret=ret, val="__id__", exp_ret="second_id", equal=False)

        check_file = self.run_function("file.file_exists", [SSH_SLS_FILE])
        self.assertTrue(check_file)

    @slowTest
    def test_state_sls_wrong_id(self):
        """
        test state.sls_id when id does not exist
        """
        # check state.sls_id with test=True
        ret = self.run_function("state.sls_id", ["doesnotexist", SSH_SLS])
        assert "No matches for ID" in ret

    @slowTest
    def test_state_sls_id_with_pillar(self):
        """
        test state.sls_id with pillar data
        """
        self.run_function(
            "state.sls_id",
            ["ssh-file-test", SSH_SLS, 'pillar=\'{"test_file_suffix": "_pillar"}\''],
        )
        check_file = self.run_function(
            "file.file_exists", ["/tmp/salt_test_file_pillar"]
        )
        self.assertTrue(check_file)

    @slowTest
    def test_state_show_sls(self):
        """
        test state.show_sls with salt-ssh
        """
        ret = self.run_function("state.show_sls", [SSH_SLS])
        self._check_dict_ret(ret=ret, val="__sls__", exp_ret=SSH_SLS)

        check_file = self.run_function("file.file_exists", [SSH_SLS_FILE], wipe=False)
        self.assertFalse(check_file)

    @slowTest
    def test_state_show_top(self):
        """
        test state.show_top with salt-ssh
        """
        ret = self.run_function("state.show_top")
        self.assertEqual(ret, {"base": ["core", "master_tops_test"]})

    @slowTest
    def test_state_single(self):
        """
        state.single with salt-ssh
        """
        ret_out = {"name": "itworked", "result": True, "comment": "Success!"}

        single = self.run_function(
            "state.single", ["test.succeed_with_changes name=itworked"]
        )

        self.assertIsInstance(single, dict)
        for key, value in six.iteritems(single):
            self.assertIsInstance(value, dict)
            self.assertEqual(value["name"], ret_out["name"])
            self.assertEqual(value["result"], ret_out["result"])
            self.assertEqual(value["comment"], ret_out["comment"])

    @slowTest
    def test_show_highstate(self):
        """
        state.show_highstate with salt-ssh
        """
        high = self.run_function("state.show_highstate")
        destpath = os.path.join(RUNTIME_VARS.TMP, "testfile")
        self.assertIsInstance(high, dict)
        self.assertIn(destpath, high)
        self.assertEqual(high[destpath]["__env__"], "base")

    @slowTest
    def test_state_high(self):
        """
        state.high with salt-ssh
        """
        ret_out = {"name": "itworked", "result": True, "comment": "Success!"}

        high = self.run_function(
            "state.high", ['"{"itworked": {"test": ["succeed_with_changes"]}}"']
        )

        self.assertIsInstance(high, dict)
        for key, value in six.iteritems(high):
            self.assertIsInstance(value, dict)
            self.assertEqual(value["name"], ret_out["name"])
            self.assertEqual(value["result"], ret_out["result"])
            self.assertEqual(value["comment"], ret_out["comment"])

    @slowTest
    def test_show_lowstate(self):
        """
        state.show_lowstate with salt-ssh
        """
        low = self.run_function("state.show_lowstate")
        self.assertIsInstance(low, list)
        self.assertIsInstance(low[0], dict)

    @slowTest
    def test_state_low(self):
        """
        state.low with salt-ssh
        """
        ret_out = {"name": "itworked", "result": True, "comment": "Success!"}

        low = self.run_function(
            "state.low",
            ['"{"state": "test", "fun": "succeed_with_changes", "name": "itworked"}"'],
        )

        self.assertIsInstance(low, dict)
        for key, value in six.iteritems(low):
            self.assertIsInstance(value, dict)
            self.assertEqual(value["name"], ret_out["name"])
            self.assertEqual(value["result"], ret_out["result"])
            self.assertEqual(value["comment"], ret_out["comment"])

    @slowTest
    def test_state_request_check_clear(self):
        """
        test state.request system with salt-ssh
        while also checking and clearing request
        """
        request = self.run_function("state.request", [SSH_SLS], wipe=False)
        self._check_dict_ret(ret=request, val="__sls__", exp_ret=SSH_SLS)

        self._check_request()

        clear = self.run_function("state.clear_request", wipe=False)
        self._check_request(empty=True)

    @slowTest
    def test_state_run_request(self):
        """
        test state.request system with salt-ssh
        while also running the request later
        """
        request = self.run_function("state.request", [SSH_SLS], wipe=False)
        self._check_dict_ret(ret=request, val="__sls__", exp_ret=SSH_SLS)

        run = self.run_function("state.run_request", wipe=False)

        check_file = self.run_function("file.file_exists", [SSH_SLS_FILE], wipe=False)
        self.assertTrue(check_file)

    @flaky
    @slowTest
    def test_state_running(self):
        """
        test state.running with salt-ssh
        """

        def _run_in_background():
            self.run_function("state.sls", ["running"], wipe=False)

        bg_thread = threading.Thread(target=_run_in_background)
        bg_thread.start()

        expected = 'The function "state.pkg" is running as'
        state_ret = []
        for _ in range(30):
            time.sleep(5)
            get_sls = self.run_function("state.running", wipe=False)
            state_ret.append(get_sls)
            if expected in " ".join(get_sls):
                # We found the expected return
                break
        else:
            self.fail(
                "Did not find '{0}' in state.running return: {1}".format(
                    expected, state_ret
                )
            )

        # make sure we wait until the earlier state is complete
        future = time.time() + 120
        while True:
            if expected not in " ".join(self.run_function("state.running", wipe=False)):
                break
            if time.time() > future:
                self.fail(
                    "state.pkg is still running overtime. Test did not clean up correctly."
                )

    def tearDown(self):
        """
        make sure to clean up any old ssh directories
        """
        salt_dir = self.run_function("config.get", ["thin_dir"], wipe=False)
        self.assertIsInstance(salt_dir, six.string_types)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)

        for test_file_path in glob.glob(SSH_SLS_FILE + "*"):
            os.remove(test_file_path)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.schedule as schedule

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ScheduleTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.schedule
    """

    def setup_loader_modules(self):
        return {schedule: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure a job is present in the schedule.
        """
        name = "job1"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_dict = MagicMock(side_effect=[ret, []])
        mock_mod = MagicMock(return_value=ret)
        mock_lst = MagicMock(side_effect=[{name: {}}, {name: {}}, {}, {}])
        with patch.dict(
            schedule.__salt__,
            {
                "schedule.list": mock_lst,
                "schedule.build_schedule_item": mock_dict,
                "schedule.modify": mock_mod,
                "schedule.add": mock_mod,
            },
        ):
            self.assertDictEqual(schedule.present(name), ret)

            with patch.dict(schedule.__opts__, {"test": False}):
                self.assertDictEqual(schedule.present(name), ret)

                self.assertDictEqual(schedule.present(name), ret)

            with patch.dict(schedule.__opts__, {"test": True}):
                ret.update({"result": True})
                self.assertDictEqual(schedule.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure a job is absent from the schedule.
        """
        name = "job1"

        ret = {"name": name, "changes": {}, "result": False, "comment": ""}

        mock_mod = MagicMock(return_value=ret)
        mock_lst = MagicMock(side_effect=[{name: {}}, {}])
        with patch.dict(
            schedule.__salt__, {"schedule.list": mock_lst, "schedule.delete": mock_mod}
        ):
            with patch.dict(schedule.__opts__, {"test": False}):
                self.assertDictEqual(schedule.absent(name), ret)

            with patch.dict(schedule.__opts__, {"test": True}):
                comt = "Job job1 not present in schedule"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(schedule.absent(name), ret)



# -*- coding: utf-8 -*-

# Import Pytohn libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Module
import salt.modules.nginx as nginx

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock, patch
from tests.support.unit import TestCase

MOCK_STATUS_OUTPUT = """Active connections: 7
server accepts handled requests
 46756 46756 89318
Reading: 0 Writing: 7 Waiting: 0"""


class MockUrllibStatus(object):
    """Mock of urllib2 call for Nginx status"""

    def read(self):
        return MOCK_STATUS_OUTPUT

    def close(self):
        pass


class NginxTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        patcher = patch("salt.utils.path.which", Mock(return_value="/usr/bin/nginx"))
        patcher.start()
        self.addCleanup(patcher.stop)
        return {nginx: {"_urlopen": Mock(return_value=MockUrllibStatus())}}

    def test_nginx_status(self):
        result = nginx.status()
        nginx._urlopen.assert_called_once_with("http://127.0.0.1/status")
        self.assertEqual(
            result,
            {
                "active connections": 7,
                "accepted": 46756,
                "handled": 46756,
                "requests": 89318,
                "reading": 0,
                "writing": 7,
                "waiting": 0,
            },
        )

    def test_nginx_status_with_arg(self):
        other_path = "http://localhost/path"
        result = nginx.status(other_path)
        nginx._urlopen.assert_called_once_with(other_path)
